@@ -23,18 +23,20 @@
 #include <caml/memory.h>
 #include <caml/mlvalues.h>
 
-#if HAVE_YAJL
 #include <yajl/yajl_tree.h>
-#endif
 
 #include <stdio.h>
 #include <string.h>
 
+/* GCC can't work out that the YAJL_IS_<foo> test is sufficient to
+ * ensure that YAJL_GET_<foo> later doesn't return NULL.
+ */
+#if defined(__GNUC__) && __GNUC__ >= 6 /* gcc >= 6 */
+#pragma GCC diagnostic ignored "-Wnull-dereference"
+#endif
+
 #define Val_none (Val_int (0))
 
-value virt_builder_yajl_is_available (value unit);
-
-#if HAVE_YAJL
 value virt_builder_yajl_tree_parse (value stringv);
 
 static value
@@ -94,13 +96,6 @@ convert_yajl_value (yajl_val val, int level)
   CAMLreturn (rv);
 }
 
-value
-virt_builder_yajl_is_available (value unit)
-{
-  /* NB: noalloc */
-  return Val_true;
-}
-
 value
 virt_builder_yajl_tree_parse (value stringv)
 {
@@ -124,21 +119,3 @@ virt_builder_yajl_tree_parse (value stringv)
 
   CAMLreturn (rv);
 }
-
-#else
-value virt_builder_yajl_tree_parse (value stringv)  __attribute__((noreturn));
-
-value
-virt_builder_yajl_is_available (value unit)
-{
-  /* NB: noalloc */
-  return Val_false;
-}
-
-value
-virt_builder_yajl_tree_parse (value stringv)
-{
-  caml_invalid_argument ("virt-builder was compiled without yajl support");
-}
-
-#endif
