@@ -35,7 +35,7 @@ GUESTFSD_EXT_CMD(str_base64, base64);
 static int
 write_cb (void *fd_ptr, const void *buf, size_t len)
 {
-  int fd = *(int *)fd_ptr;
+  const int fd = *(int *)fd_ptr;
   return xwrite (fd, buf, len);
 }
 
@@ -106,7 +106,13 @@ do_base64_out (const char *file)
   int r;
   FILE *fp;
   CLEANUP_FREE char *cmd = NULL;
-  char buffer[GUESTFS_MAX_CHUNK_SIZE];
+  CLEANUP_FREE char *buffer = NULL;
+
+  buffer = malloc (GUESTFS_MAX_CHUNK_SIZE);
+  if (buffer == NULL) {
+    reply_with_perror ("malloc");
+    return -1;
+  }
 
   /* Check the filename exists and is not a directory (RHBZ#908322). */
   buf = sysroot_path (file);
@@ -146,7 +152,7 @@ do_base64_out (const char *file)
    */
   reply (NULL, NULL);
 
-  while ((r = fread (buffer, 1, sizeof buffer, fp)) > 0) {
+  while ((r = fread (buffer, 1, GUESTFS_MAX_CHUNK_SIZE, fp)) > 0) {
     if (send_file_write (buffer, r) < 0) {
       pclose (fp);
       return -1;
