@@ -23,6 +23,8 @@
 #include <inttypes.h>
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
+#include <error.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 
@@ -35,70 +37,67 @@
 #include "actions.h"
 
 GUESTFSD_EXT_CMD(str_lvm, lvm);
-GUESTFSD_EXT_CMD(str_cp, cp);
 GUESTFSD_EXT_CMD(str_rm, rm);
+GUESTFSD_EXT_CMD(str_lvmetad, lvmetad);
 
-/* This runs during daemon start up and creates a complete copy of
- * /etc/lvm so that we can modify it as we desire.  We set
- * LVM_SYSTEM_DIR to point to the copy.  Note that the final directory
- * layout is:
+/* This runs during daemon start up and creates a fresh LVM
+ * configuration which we can modify as we desire.  LVM allows
+ * configuration to be completely empty (meaning "all defaults").
+ *
+ * The final directory layout is:
+ *
  *   /tmp/lvmXXXXXX                 (lvm_system_dir set to this)
  *   /tmp/lvmXXXXXX/lvm             ($LVM_SYSTEM_DIR set to this)
- *   /tmp/lvmXXXXXX/lvm/lvm.conf    (configuration file)
- *   /tmp/lvmXXXXXX/lvm/cache
- *   etc.
+ *   /tmp/lvmXXXXXX/lvm/lvm.conf    (configuration file - initially empty)
  */
 static char lvm_system_dir[] = "/tmp/lvmXXXXXX";
 
 static void rm_lvm_system_dir (void);
+static void debug_lvm_config (void);
 
 void
-copy_lvm (void)
+clean_lvm_config (void)
 {
-  struct stat statbuf;
-  char cmd[64], env[64];
-  int r;
+  char env[64], conf[64];
+  FILE *fp;
 
-  /* If /etc/lvm directory doesn't exist (or isn't a directory) assume
-   * that this system doesn't support LVM and do nothing.
-   */
-  r = stat ("/etc/lvm", &statbuf);
-  if (r == -1) {
-    perror ("copy_lvm: stat: /etc/lvm");
-    return;
-  }
-  if (! S_ISDIR (statbuf.st_mode)) {
-    fprintf (stderr, "copy_lvm: warning: /etc/lvm is not a directory\n");
-    return;
-  }
+  if (mkdtemp (lvm_system_dir) == NULL)
+    error (EXIT_FAILURE, errno, "mkdtemp: %s", lvm_system_dir);
 
-  if (mkdtemp (lvm_system_dir) == NULL) {
-    fprintf (stderr, "mkdtemp: %s: %m\n", lvm_system_dir);
-    exit (EXIT_FAILURE);
-  }
-
-  /* Copy the entire directory */
-  snprintf (cmd, sizeof cmd, "%s -a /etc/lvm/ %s", str_cp, lvm_system_dir);
-  r = system (cmd);
-  if (r == -1) {
-    perror (cmd);
-    rmdir (lvm_system_dir);
-    exit (EXIT_FAILURE);
-  }
-
-  if (WEXITSTATUS (r) != 0) {
-    fprintf (stderr, "cp command failed with return code %d\n",
-             WEXITSTATUS (r));
-    rmdir (lvm_system_dir);
-    exit (EXIT_FAILURE);
-  }
-
-  /* Set environment variable so we use the copy. */
   snprintf (env, sizeof env, "%s/lvm", lvm_system_dir);
+  mkdir (env, 0755);
+  snprintf (conf, sizeof conf, "%s/lvm/lvm.conf", lvm_system_dir);
+  fp = fopen (conf, "w");
+  if (fp == NULL) {
+    perror ("clean_lvm_config: cannot create empty lvm.conf");
+    exit (EXIT_FAILURE);
+  }
+  fclose (fp);
+
+  /* Set environment variable so we use the clean configuration. */
   setenv ("LVM_SYSTEM_DIR", env, 1);
 
   /* Set a handler to remove the temporary directory at exit. */
   atexit (rm_lvm_system_dir);
+
+  debug_lvm_config ();
+}
+
+/* Try to run lvmetad, without failing if it couldn't. */
+void
+start_lvmetad (void)
+{
+  char cmd[64];
+  int r;
+
+  snprintf (cmd, sizeof cmd, "%s", str_lvmetad);
+  if (verbose)
+    printf ("%s\n", cmd);
+  r = system (cmd);
+  if (r == -1)
+    perror ("system/lvmetad");
+  else if (!WIFEXITED (r) || WEXITSTATUS (r) != 0)
+    fprintf (stderr, "warning: lvmetad command failed\n");
 }
 
 static void
@@ -114,74 +113,40 @@ rm_lvm_system_dir (void)
 static int
 set_filter (char *const *filters)
 {
-  CLEANUP_AUG_CLOSE augeas *aug = NULL;
-  int r;
-  int count;
+  const char *filter_types[] = { "filter", "global_filter", NULL };
+  CLEANUP_FREE char *conf = NULL;
+  FILE *fp;
+  size_t i, j;
 
-  /* Small optimization: do not load the files at init time,
-   * but do that only after having applied the transformation.
-   */
-  const int flags = AUG_NO_ERR_CLOSE | AUG_NO_LOAD;
-  aug = aug_init (lvm_system_dir, "/usr/share/guestfs/", flags);
-  if (!aug) {
-    reply_with_error ("augeas initialization failed");
+  if (asprintf (&conf, "%s/lvm/lvm.conf", lvm_system_dir) == -1) {
+    reply_with_perror ("asprintf");
+    return -1;
+  }
+  fp = fopen (conf, "we");
+  if (fp == NULL) {
+    reply_with_perror ("open: %s", conf);
     return -1;
   }
 
-  if (aug_error (aug) != AUG_NOERROR) {
-    AUGEAS_ERROR ("aug_init");
-    return -1;
-  }
+  fprintf (fp, "devices {\n");
+  for (j = 0; filter_types[j] != NULL; ++j) {
+    fprintf (fp, "    %s = [\n", filter_types[j]);
+    fprintf (fp, "        ");
 
-  r = aug_transform (aug, "guestfs_lvm_conf", "/lvm/lvm.conf",
-                     0 /* = included */);
-  if (r == -1) {
-    AUGEAS_ERROR ("aug_transform");
-    return -1;
-  }
-
-  if (aug_load (aug) == -1) {
-    AUGEAS_ERROR ("aug_load");
-    return -1;
-  }
-
-  /* Remove all the old filters ... */
-  r = aug_rm (aug, "/files/lvm/lvm.conf/devices/dict/filter/list/*");
-  if (r == -1) {
-    AUGEAS_ERROR ("aug_rm");
-    return -1;
-  }
-
-  /* ... and add the new ones. */
-  for (count = 0; filters[count] != NULL; ++count) {
-    char buf[128];
-
-    snprintf (buf, sizeof buf,
-              "/files/lvm/lvm.conf/devices/dict/filter/list/%d/str",
-              count + 1);
-
-    if (aug_set (aug, buf, filters[count]) == -1) {
-      AUGEAS_ERROR ("aug_set: %d: %s", count, filters[count]);
-      return -1;
+    for (i = 0; filters[i] != NULL; ++i) {
+      if (i > 0)
+        fprintf (fp, ",\n        ");
+      fprintf (fp, "\"%s\"", filters[i]);
     }
-  }
 
-  /* Safety check for the written filter nodes. */
-  r = aug_match (aug, "/files/lvm/lvm.conf/devices/dict/filter/list/*/str",
-                 NULL);
-  if (r == -1) {
-    AUGEAS_ERROR ("aug_match");
-    return -1;
-  }
-  if (r != count) {
-    reply_with_error ("filters# vs matches mismatch: %d vs %d", count, r);
-    return -1;
+    fprintf (fp, "\n");
+    fprintf (fp, "    ]\n");
   }
+  fprintf (fp, "}\n");
 
-  if (aug_save (aug) == -1) {
-    AUGEAS_ERROR ("aug_save");
-    return -1;
-  }
+  fclose (fp);
+
+  debug_lvm_config ();
 
   return 0;
 }
@@ -232,6 +197,16 @@ rescan (void)
   return 0;
 }
 
+/* Show what lvm thinks is the current config.  Useful for debugging. */
+static void
+debug_lvm_config (void)
+{
+  if (verbose) {
+    fprintf (stderr, "lvm config:\n");
+    ignore_value (system ("lvm config"));
+  }
+}
+
 /* Construct the new, specific filter strings.  We can assume that
  * the 'devices' array does not contain any regexp metachars,
  * because it's already been checked by the stub code.
