@@ -22,6 +22,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
+#include <error.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <mntent.h>
@@ -49,10 +51,8 @@ is_root_mounted (void)
    * that requires custom parsing code.
    */
   fp = setmntent ("/proc/mounts", "r");
-  if (fp == NULL) {
-    fprintf (stderr, "setmntent: %s: %m\n", "/proc/mounts");
-    exit (EXIT_FAILURE);
-  }
+  if (fp == NULL)
+    error (EXIT_FAILURE, errno, "setmntent: %s", "/proc/mounts");
 
   while ((m = getmntent (fp)) != NULL) {
     /* Allow a mount directory like "/sysroot". */
@@ -91,10 +91,8 @@ is_device_mounted (const char *device)
    * that requires custom parsing code.
    */
   fp = setmntent ("/proc/mounts", "r");
-  if (fp == NULL) {
-    fprintf (stderr, "setmntent: %s: %m\n", "/proc/mounts");
-    exit (EXIT_FAILURE);
-  }
+  if (fp == NULL)
+    error (EXIT_FAILURE, errno, "setmntent: %s", "/proc/mounts");
 
   while ((m = getmntent (fp)) != NULL) {
     if ((sysroot_len > 0 && STREQ (m->mnt_dir, sysroot)) ||
@@ -129,7 +127,7 @@ do_mount_vfs (const char *options, const char *vfstype,
   CLEANUP_FREE char *mp = NULL;
   struct stat statbuf;
 
-  ABS_PATH (mountpoint, , return -1);
+  ABS_PATH (mountpoint, 0, return -1);
 
   mp = sysroot_path (mountpoint);
   if (!mp) {
@@ -274,10 +272,8 @@ mounts_or_mountpoints (int mp)
    * that requires custom parsing code.
    */
   fp = setmntent ("/proc/mounts", "r");
-  if (fp == NULL) {
-    fprintf (stderr, "setmntent: %s: %m\n", "/proc/mounts");
-    exit (EXIT_FAILURE);
-  }
+  if (fp == NULL)
+    error (EXIT_FAILURE, errno, "setmntent: %s", "/proc/mounts");
 
   while ((m = getmntent (fp)) != NULL) {
     /* Allow a mount directory like "/sysroot". */
@@ -381,10 +377,8 @@ do_umount_all (void)
    * that requires custom parsing code.
    */
   fp = setmntent ("/proc/mounts", "r");
-  if (fp == NULL) {
-    fprintf (stderr, "setmntent: %s: %m\n", "/proc/mounts");
-    exit (EXIT_FAILURE);
-  }
+  if (fp == NULL)
+    error (EXIT_FAILURE, errno, "setmntent: %s", "/proc/mounts");
 
   while ((m = getmntent (fp)) != NULL) {
     if (verbose) {
@@ -504,8 +498,8 @@ do_mkmountpoint (const char *path)
 {
   int r;
 
-  /* NEED_ROOT (return -1); - we don't want this test for this call. */
-  ABS_PATH (path, , return -1);
+  /* NEED_ROOT (0, return -1); - we don't want this test for this call. */
+  ABS_PATH (path, 0, return -1);
 
   CHROOT_IN;
   r = mkdir (path, 0777);
@@ -524,8 +518,8 @@ do_rmmountpoint (const char *path)
 {
   int r;
 
-  /* NEED_ROOT (return -1); - we don't want this test for this call. */
-  ABS_PATH (path, , return -1);
+  /* NEED_ROOT (0, return -1); - we don't want this test for this call. */
+  ABS_PATH (path, 0, return -1);
 
   CHROOT_IN;
   r = rmdir (path);
