@@ -1,5 +1,5 @@
 /* libguestfs - the guestfsd daemon
- * Copyright (C) 2009-2014 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,15 @@
 #include "actions.h"
 #include "optgroups.h"
 
-#if defined(HAVE_ATTR_XATTR_H) || defined(HAVE_SYS_XATTR_H)
+#if (defined(HAVE_ATTR_XATTR_H) || defined(HAVE_SYS_XATTR_H)) &&	\
+  defined(HAVE_LISTXATTR) && defined(HAVE_LLISTXATTR) &&		\
+  defined(HAVE_GETXATTR) && defined(HAVE_LGETXATTR) &&			\
+  defined(HAVE_REMOVEXATTR) && defined(HAVE_LREMOVEXATTR) &&		\
+  defined(HAVE_SETXATTR) && defined(HAVE_LSETXATTR)
+# define HAVE_LINUX_XATTRS
+#endif
+
+#ifdef HAVE_LINUX_XATTRS
 
 # ifdef HAVE_ATTR_XATTR_H
 #  include <attr/xattr.h>
@@ -46,71 +54,52 @@ optgroup_linuxxattrs_available (void)
 static guestfs_int_xattr_list *getxattrs (const char *path, ssize_t (*listxattr) (const char *path, char *list, size_t size), ssize_t (*getxattr) (const char *path, const char *name, void *value, size_t size));
 static int _setxattr (const char *xattr, const char *val, int vallen, const char *path, int (*setxattr) (const char *path, const char *name, const void *value, size_t size, int flags));
 static int _removexattr (const char *xattr, const char *path, int (*removexattr) (const char *path, const char *name));
+static char *_listxattrs (const char *path, ssize_t (*listxattr) (const char *path, char *list, size_t size), ssize_t *size);
+static char *_getxattr (const char *name, const char *path, ssize_t (*getxattr) (const char *path, const char *name, void *value, size_t size), size_t *size_r);
 
 guestfs_int_xattr_list *
 do_getxattrs (const char *path)
 {
-#if defined(HAVE_LISTXATTR) && defined(HAVE_GETXATTR)
   return getxattrs (path, listxattr, getxattr);
-#else
-  reply_with_error ("no support for listxattr and getxattr");
-  return NULL;
-#endif
 }
 
 guestfs_int_xattr_list *
 do_lgetxattrs (const char *path)
 {
-#if defined(HAVE_LLISTXATTR) && defined(HAVE_LGETXATTR)
   return getxattrs (path, llistxattr, lgetxattr);
-#else
-  reply_with_error ("no support for llistxattr and lgetxattr");
-  return NULL;
-#endif
 }
 
 int
 do_setxattr (const char *xattr, const char *val, int vallen, const char *path)
 {
-#if defined(HAVE_SETXATTR)
   return _setxattr (xattr, val, vallen, path, setxattr);
-#else
-  reply_with_error ("no support for setxattr");
-  return -1;
-#endif
 }
 
 int
 do_lsetxattr (const char *xattr, const char *val, int vallen, const char *path)
 {
-#if defined(HAVE_LSETXATTR)
   return _setxattr (xattr, val, vallen, path, lsetxattr);
-#else
-  reply_with_error ("no support for lsetxattr");
-  return -1;
-#endif
 }
 
 int
 do_removexattr (const char *xattr, const char *path)
 {
-#if defined(HAVE_REMOVEXATTR)
   return _removexattr (xattr, path, removexattr);
-#else
-  reply_with_error ("no support for removexattr");
-  return -1;
-#endif
 }
 
 int
 do_lremovexattr (const char *xattr, const char *path)
 {
-#if defined(HAVE_LREMOVEXATTR)
   return _removexattr (xattr, path, lremovexattr);
-#else
-  reply_with_error ("no support for lremovexattr");
-  return -1;
-#endif
+}
+
+static int
+compare_xattrs (const void *vxa1, const void *vxa2)
+{
+  const guestfs_int_xattr *xa1 = vxa1;
+  const guestfs_int_xattr *xa2 = vxa2;
+
+  return strcmp (xa1->attrname, xa2->attrname);
 }
 
 static guestfs_int_xattr_list *
@@ -124,31 +113,14 @@ getxattrs (const char *path,
   size_t i, j;
   guestfs_int_xattr_list *r = NULL;
 
-  CHROOT_IN;
-  len = listxattr (path, NULL, 0);
-  CHROOT_OUT;
-  if (len == -1) {
-    reply_with_perror ("listxattr: %s", path);
+  buf = _listxattrs (path, listxattr, &len);
+  if (buf == NULL)
+    /* _listxattrs issues reply_with_perror already. */
     goto error;
-  }
-
-  buf = malloc (len);
-  if (buf == NULL) {
-    reply_with_perror ("malloc");
-    goto error;
-  }
-
-  CHROOT_IN;
-  len = listxattr (path, buf, len);
-  CHROOT_OUT;
-  if (len == -1) {
-    reply_with_perror ("listxattr: %s", path);
-    goto error;
-  }
 
   r = calloc (1, sizeof (*r));
   if (r == NULL) {
-    reply_with_perror ("malloc");
+    reply_with_perror ("calloc");
     goto error;
   }
 
@@ -202,6 +174,12 @@ getxattrs (const char *path,
     }
   }
 
+  /* Sort the entries by attrname. */
+  qsort (&r->guestfs_int_xattr_list_val[0],
+         (size_t) r->guestfs_int_xattr_list_len,
+         sizeof (guestfs_int_xattr),
+         compare_xattrs);
+
   return r;
 
  error:
@@ -259,10 +237,48 @@ _removexattr (const char *xattr, const char *path,
   return 0;
 }
 
+static char *
+_listxattrs (const char *path,
+             ssize_t (*listxattr) (const char *path, char *list, size_t size),
+             ssize_t *size)
+{
+  char *buf = NULL;
+  ssize_t len;
+
+  CHROOT_IN;
+  len = listxattr (path, NULL, 0);
+  CHROOT_OUT;
+  if (len == -1) {
+    reply_with_perror ("listxattr: %s", path);
+    goto error;
+  }
+
+  buf = malloc (len);
+  if (buf == NULL) {
+    reply_with_perror ("malloc");
+    goto error;
+  }
+
+  CHROOT_IN;
+  len = listxattr (path, buf, len);
+  CHROOT_OUT;
+  if (len == -1) {
+    reply_with_perror ("listxattr: %s", path);
+    goto error;
+  }
+
+  if (size)
+    *size = len;
+  return buf;
+
+ error:
+  free (buf);
+  return NULL;
+}
+
 guestfs_int_xattr_list *
 do_internal_lxattrlist (const char *path, char *const *names)
 {
-#if defined(HAVE_LLISTXATTR) && defined(HAVE_LGETXATTR)
   guestfs_int_xattr_list *ret = NULL;
   size_t i, j;
   size_t k, m, nr_attrs;
@@ -360,7 +376,7 @@ do_internal_lxattrlist (const char *path, char *const *names)
      * entry[1..nr_attrs] are the attributes.
      */
     entry = &ret->guestfs_int_xattr_list_val[ret->guestfs_int_xattr_list_len-nr_attrs-1];
-    for (m = 1; m <= nr_attrs; ++m) {
+    for (m = 1; m < nr_attrs+1; ++m) {
       entry[m].attrname = NULL;
       entry[m].attrval.attrval_len = 0;
       entry[m].attrval.attrval_val = NULL;
@@ -409,6 +425,9 @@ do_internal_lxattrlist (const char *path, char *const *names)
       reply_with_perror ("strdup");
       goto error;
     }
+
+    /* Sort the entries by attrname (only for this single file). */
+    qsort (&entry[1], nr_attrs, sizeof (guestfs_int_xattr), compare_xattrs);
   }
 
   return ret;
@@ -425,14 +444,19 @@ do_internal_lxattrlist (const char *path, char *const *names)
     free (ret);
   }
   return NULL;
-#else
-  reply_with_error ("no support for llistxattr and lgetxattr");
-  return NULL;
-#endif
 }
 
 char *
 do_getxattr (const char *path, const char *name, size_t *size_r)
+{
+  return _getxattr (name, path, getxattr, size_r);
+}
+
+static char *
+_getxattr (const char *name, const char *path,
+           ssize_t (*getxattr) (const char *path, const char *name,
+                                void *value, size_t size),
+           size_t *size_r)
 {
   ssize_t r;
   char *buf;
@@ -482,53 +506,80 @@ do_getxattr (const char *path, const char *name, size_t *size_r)
 char *
 do_lgetxattr (const char *path, const char *name, size_t *size_r)
 {
-  ssize_t r;
-  char *buf;
-  size_t len;
-
-  CHROOT_IN;
-  r = lgetxattr (path, name, NULL, 0);
-  CHROOT_OUT;
-  if (r == -1) {
-    reply_with_perror ("lgetxattr");
-    return NULL;
-  }
-
-  len = r;
-
-  if (len > XATTR_SIZE_MAX) {
-    reply_with_error ("extended attribute is too large");
-    return NULL;
-  }
-
-  buf = malloc (len);
-  if (buf == NULL) {
-    reply_with_perror ("malloc");
-    return NULL;
-  }
-
-  CHROOT_IN;
-  r = lgetxattr (path, name, buf, len);
-  CHROOT_OUT;
-  if (r == -1) {
-    reply_with_perror ("lgetxattr");
-    free (buf);
-    return NULL;
-  }
-
-  if (len != (size_t) r) {
-    reply_with_error ("lgetxattr: unexpected size (%zu/%zd)", len, r);
-    free (buf);
-    return NULL;
-  }
-
-  /* Must set size_r last thing before returning. */
-  *size_r = len;
-  return buf; /* caller frees */
+  return _getxattr (name, path, lgetxattr, size_r);
 }
 
-#else /* no xattr.h */
+int
+copy_xattrs (const char *src, const char *dest)
+{
+  ssize_t len, vlen, ret, attrval_len = 0;
+  CLEANUP_FREE char *buf = NULL, *attrval = NULL;
+  size_t i;
+
+  buf = _listxattrs (src, listxattr, &len);
+  if (buf == NULL)
+    /* _listxattrs issues reply_with_perror already. */
+    goto error;
+
+  /* What we get from the kernel is a string "foo\0bar\0baz" of length
+   * len.
+   */
+  for (i = 0; i < (size_t) len; i += strlen (&buf[i]) + 1) {
+    CHROOT_IN;
+    vlen = getxattr (src, &buf[i], NULL, 0);
+    CHROOT_OUT;
+    if (vlen == -1) {
+      reply_with_perror ("getxattr: %s, %s", src, &buf[i]);
+      goto error;
+    }
+
+    if (vlen > XATTR_SIZE_MAX) {
+      /* The next call to getxattr will fail anyway, so ... */
+      reply_with_error ("extended attribute is too large");
+      goto error;
+    }
+
+    if (vlen > attrval_len) {
+      char *new = realloc (attrval, vlen);
+      if (new == NULL) {
+        reply_with_perror ("realloc");
+        goto error;
+      }
+      attrval = new;
+      attrval_len = vlen;
+    }
+
+    CHROOT_IN;
+    vlen = getxattr (src, &buf[i], attrval, vlen);
+    CHROOT_OUT;
+    if (vlen == -1) {
+      reply_with_perror ("getxattr: %s, %s", src, &buf[i]);
+      goto error;
+    }
+
+    CHROOT_IN;
+    ret = setxattr (dest, &buf[i], attrval, vlen, 0);
+    CHROOT_OUT;
+    if (ret == -1) {
+      reply_with_perror ("setxattr: %s, %s", dest, &buf[i]);
+      goto error;
+    }
+  }
+
+  return 1;
+
+ error:
+  return 0;
+}
+
+#else /* no HAVE_LINUX_XATTRS */
 
 OPTGROUP_LINUXXATTRS_NOT_AVAILABLE
 
-#endif /* no xattr.h */
+int
+copy_xattrs (const char *src, const char *dest)
+{
+  abort ();
+}
+
+#endif /* no HAVE_LINUX_XATTRS */
