@@ -16,7 +16,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-export LANG=C
 set -e
 
-$srcdir/../podcheck.pl virt-edit.pod virt-edit
+$TEST_FUNCTIONS
+
+$top_srcdir/podcheck.pl virt-edit.pod virt-edit
