@@ -16,10 +16,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-export LANG=C
 set -e
 
-$srcdir/../podcheck.pl guestfish.pod guestfish \
+$TEST_FUNCTIONS
+skip_if_skipped
+
+$top_srcdir/podcheck.pl guestfish.pod guestfish \
     --insert $srcdir/guestfish-actions.pod:__ACTIONS__ \
     --insert $srcdir/guestfish-commands.pod:__FISH_COMMANDS__ \
     --insert $srcdir/guestfish-prepopts.pod:__PREPOPTS__ \
