@@ -24,11 +24,13 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
+#include <error.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
 #include "ignore-value.h"
+#include "getprogname.h"
 
 #include "guestfs.h"
 #include "guestfs-internal-frontend.h"
@@ -50,52 +52,36 @@ main (int argc, char *argv[])
 
   /* Allow the test to be skipped. */
   skip = getenv ("SKIP_TEST_FUSE_SH");
-  if (skip && guestfs_int_is_true (skip) > 0) {
-    fprintf (stderr, "%s: test skipped because environment variable set.\n",
-             guestfs_int_program_name);
-    exit (77);
-  }
+  if (skip && guestfs_int_is_true (skip) > 0)
+    error (77, 0, "test skipped because environment variable set");
 
   skip = getenv ("SKIP_TEST_GUESTMOUNT_FD");
-  if (skip && guestfs_int_is_true (skip) > 0) {
-    fprintf (stderr, "%s: test skipped because environment variable set.\n",
-             guestfs_int_program_name);
-    exit (77);
-  }
+  if (skip && guestfs_int_is_true (skip) > 0)
+    error (77, 0, "test skipped because environment variable set");
 
   /* Skip the test if the test image can't be found. */
-  if (access (TEST_IMAGE, R_OK) == -1) {
-    perror (TEST_IMAGE);
-    exit (77);
-  }
+  if (access (TEST_IMAGE, R_OK) == -1)
+    error (77, errno, "access: %s", TEST_IMAGE);
 
   /* Skip the test if /dev/fuse is not writable, because guestmount
    * will fail.
    */
-  if (access ("/dev/fuse", W_OK) == -1) {
-    perror ("/dev/fuse");
-    exit (77);
-  }
+  if (access ("/dev/fuse", W_OK) == -1)
+    error (77, errno, "access: %s", "/dev/fuse");
 
   /* Create the pipe. */
-  if (pipe (pipefd) == -1) {
-    perror ("pipe");
-    exit (EXIT_FAILURE);
-  }
+  if (pipe (pipefd) == -1)
+    error (EXIT_FAILURE, errno, "pipe");
 
   /* Create the mount point. */
   ignore_value (rmdir (MOUNTPOINT));
-  if (mkdir (MOUNTPOINT, 0700) == -1) {
-    perror ("mkdir: " MOUNTPOINT);
-    exit (EXIT_FAILURE);
-  }
+  if (mkdir (MOUNTPOINT, 0700) == -1)
+    error (EXIT_FAILURE, errno, "mkdir: %s", MOUNTPOINT);
 
   /* Create the guestmount subprocess. */
   pid = fork ();
-  if (pid == -1) {
-    perror ("fork");
-    exit (EXIT_FAILURE);
-  }
+  if (pid == -1)
+    error (EXIT_FAILURE, errno, "fork");
 
   if (pid == 0) {               /* child - guestmount */
     char fd_str[64];
@@ -124,7 +110,7 @@ main (int argc, char *argv[])
   }
   if (r == 0) {
     fprintf (stderr, "%s: unexpected end of file on pipe fd.\n",
-             guestfs_int_program_name);
+             getprogname ());
     ignore_value (rmdir (MOUNTPOINT));
     exit (EXIT_FAILURE);
   }
@@ -132,7 +118,7 @@ main (int argc, char *argv[])
   /* Check that the test image was mounted. */
   if (access (TEST_FILE, R_OK) == -1) {
     fprintf (stderr, "%s: test failed because test image is not mounted and ready.",
-             guestfs_int_program_name);
+             getprogname ());
     ignore_value (rmdir (MOUNTPOINT));
     exit (EXIT_FAILURE);
   }
@@ -142,7 +128,7 @@ main (int argc, char *argv[])
   if (r != 0) {
     char status_string[80];
 
-    fprintf (stderr, "%s: test failed: %s\n", guestfs_int_program_name,
+    fprintf (stderr, "%s: test failed: %s\n", getprogname (),
              guestfs_int_exit_status_to_string (r, GUESTUNMOUNT_BINARY,
 						status_string,
 						sizeof status_string));
@@ -163,7 +149,7 @@ main (int argc, char *argv[])
     char status_string[80];
 
     fprintf (stderr, "%s: test failed: %s\n",
-             guestfs_int_program_name,
+             getprogname (),
              guestfs_int_exit_status_to_string (status, GUESTMOUNT_BINARY,
 						status_string,
 						sizeof status_string));
