@@ -28,9 +28,11 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
+#include <error.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#include "cloexec.h"
 #include "ignore-value.h"
 
 #include "guestfs.h"
@@ -46,31 +48,21 @@ main (int argc, char *argv[])
 
   /* Allow the test to be skipped. */
   skip = getenv ("SKIP_TEST_FUSE_SH");
-  if (skip && guestfs_int_is_true (skip) > 0) {
-    fprintf (stderr, "%s: test skipped because environment variable set.\n",
-             guestfs_int_program_name);
-    exit (77);
-  }
+  if (skip && guestfs_int_is_true (skip) > 0)
+    error (77, 0, "test skipped because environment variable set");
 
   skip = getenv ("SKIP_TEST_GUESTUNMOUNT_FD");
-  if (skip && guestfs_int_is_true (skip) > 0) {
-    fprintf (stderr, "%s: test skipped because environment variable set.\n",
-             guestfs_int_program_name);
-    exit (77);
-  }
+  if (skip && guestfs_int_is_true (skip) > 0)
+    error (77, 0, "test skipped because environment variable set");
 
   /* Create the pipe. */
-  if (pipe (pipefd) == -1) {
-    perror ("pipe");
-    exit (EXIT_FAILURE);
-  }
+  if (pipe (pipefd) == -1)
+    error (EXIT_FAILURE, errno, "pipe");
 
   /* Create the guestunmount subprocess. */
   pid = fork ();
-  if (pid == -1) {
-    perror ("fork");
-    exit (EXIT_FAILURE);
-  }
+  if (pid == -1)
+    error (EXIT_FAILURE, errno, "fork");
 
   if (pid == 0) {               /* child - guestunmount */
     char fd_str[64];
@@ -86,24 +78,22 @@ main (int argc, char *argv[])
 
   /* Parent continues. */
   close (pipefd[0]);
-  ignore_value (fcntl (pipefd[1], F_SETFD, FD_CLOEXEC));
+  ignore_value (set_cloexec_flag (pipefd[1], 1));
 
   /* Sleep a bit and test that the guestunmount process is still running. */
   sleep (2);
 
   r = waitpid (pid, &status, WNOHANG);
-  if (r == -1) {
-    perror ("waitpid");
-    exit (EXIT_FAILURE);
-  }
+  if (r == -1)
+    error (EXIT_FAILURE, errno, "waitpid");
   if (r != 0) {
     char status_string[80];
 
-    fprintf (stderr, "%s: test failed: %s\n", guestfs_int_program_name,
-             guestfs_int_exit_status_to_string (r, "guestunmount",
-						status_string,
-						sizeof status_string));
-    exit (EXIT_FAILURE);
+    error (EXIT_FAILURE, 0,
+           "test failed: %s",
+           guestfs_int_exit_status_to_string (r, "guestunmount",
+                                              status_string,
+                                              sizeof status_string));
   }
 
   /* Close the write side of the pipe.  This should cause guestunmount
@@ -113,19 +103,16 @@ main (int argc, char *argv[])
   close (pipefd[1]);
 
   r = waitpid (pid, &status, 0);
-  if (r == -1) {
-    perror ("waitpid");
-    exit (EXIT_FAILURE);
-  }
+  if (r == -1)
+    error (EXIT_FAILURE, errno, "waitpid");
   if (!WIFEXITED (status) || WEXITSTATUS (status) != 3) {
     char status_string[80];
 
-    fprintf (stderr, "%s: test failed: guestunmount didn't return status code 3; %s\n",
-             guestfs_int_program_name,
-             guestfs_int_exit_status_to_string (status, "guestunmount",
-						status_string,
-						sizeof status_string));
-    exit (EXIT_FAILURE);
+    error (EXIT_FAILURE, 0,
+           "test failed: guestunmount didn't return status code 3; %s",
+           guestfs_int_exit_status_to_string (status, "guestunmount",
+                                              status_string,
+                                              sizeof status_string));
   }
 
   exit (EXIT_SUCCESS);
