@@ -1,5 +1,5 @@
 (* libguestfs
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 open Unix
 open Printf
 
+open Common_utils
 open Types
 open Utils
 open Pr
@@ -38,10 +39,11 @@ of somewhere between 2MB and 4MB.  See L<guestfs(3)/PROTOCOL LIMITS>."
 
 let deprecation_notice ?(prefix = "") ?(replace_underscores = false) =
   function
-  | { deprecated_by = None } -> None
-  | { deprecated_by = Some alt } ->
+  | { deprecated_by = Not_deprecated } -> None
+
+  | { deprecated_by = Replaced_by alt } ->
     let alt =
-      if replace_underscores then replace_char alt '_' '-' else alt in
+      if replace_underscores then String.replace_char alt '_' '-' else alt in
     let txt =
       sprintf "I<This function is deprecated.>
 In new code, use the L</%s%s> call instead.
@@ -51,6 +53,15 @@ fact that they are deprecated indicates that there are problems
 with correct use of these functions." prefix alt in
     Some txt
 
+  | { deprecated_by = Deprecated_no_replacement } ->
+     Some "I<This function is deprecated.>
+There is no replacement.  Consult the API documentation in
+L<guestfs(3)> for further information.
+
+Deprecated functions will not be removed from the API, but the
+fact that they are deprecated indicates that there are problems
+with correct use of these functions."
+
 let version_added = function
   | { added = (0, 0, release) } -> Some (sprintf "0.%d" release)
   | { added = ((0|1) as major, minor, release) } ->
@@ -67,8 +78,7 @@ type comment_style =
   | ErlangStyle | LuaStyle | PODStyle
 type license = GPLv2plus | LGPLv2plus
 
-let generate_header ?(extra_inputs = []) ?emacs_mode comment license =
-  let inputs = "generator/ *.ml" :: extra_inputs in
+let generate_header ?(inputs = []) ?emacs_mode comment license =
   let c = match comment with
     | CStyle ->         pr "/* "; " *"
     | CPlusPlusStyle -> pr "// "; "//"
@@ -84,8 +94,14 @@ let generate_header ?(extra_inputs = []) ?emacs_mode comment license =
   | Some mode -> pr " -*- %s -*-" mode
   );
   pr "\n";
-  pr "%s WARNING: THIS FILE IS GENERATED FROM:\n" c;
-  List.iter (pr "%s   %s\n" c) inputs;
+  if inputs <> [] then (
+    pr "%s WARNING: THIS FILE IS GENERATED FROM THE FOLLOWING FILES:\n" c;
+    List.iter (pr "%s          %s\n" c) inputs;
+    pr "%s          and from the code in the generator/ subdirectory.\n" c
+  ) else (
+    pr "%s WARNING: THIS FILE IS GENERATED\n" c;
+    pr "%s          from the code in the generator/ subdirectory.\n" c
+  );
   pr "%s ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.\n" c;
   pr "%s\n" c;
   pr "%s Copyright (C) %s Red Hat Inc.\n" c copyright_years;
