@@ -20,6 +20,7 @@
 
 open Printf
 
+open Common_utils
 open Types
 open Utils
 open Pr
@@ -141,6 +142,7 @@ guestfs_int_lua_create (lua_State *L)
   guestfs_h *g;
   struct userdata *u;
   unsigned flags = 0;
+  char err[256];
 
   if (lua_gettop (L) == 1) {
     OPTARG_IF_SET (1, \"environment\",
@@ -156,9 +158,10 @@ guestfs_int_lua_create (lua_State *L)
     return luaL_error (L, \"Guestfs.create: too many arguments\");
 
   g = guestfs_create_flags (flags);
-  if (!g)
-    return luaL_error (L, \"Guestfs.create: cannot create handle: %%s\",
-                       strerror (errno));
+  if (!g) {
+    strerror_r (errno, err, sizeof err);
+    return luaL_error (L, \"Guestfs.create: cannot create handle: %%s\", err);
+  }
 
   guestfs_set_error_handler (g, NULL, NULL);
 
@@ -225,6 +228,7 @@ error__tostring (lua_State *L)
 {
   int code;
   const char *msg;
+  char err[256];
 
   lua_pushliteral (L, \"code\");
   lua_gettable (L, 1);
@@ -233,8 +237,10 @@ error__tostring (lua_State *L)
   lua_gettable (L, 1);
   msg = luaL_checkstring (L, -1);
 
-  if (code)
-    lua_pushfstring (L, \"%%s: %%s\", msg, strerror (code));
+  if (code) {
+    strerror_r (code, err, sizeof err);
+    lua_pushfstring (L, \"%%s: %%s\", msg, err);
+  }
   else
     lua_pushstring (L, msg);
 
@@ -529,7 +535,7 @@ guestfs_int_lua_delete_event_callback (lua_State *L)
         List.iter (
           fun optarg ->
             let n = name_of_optargt optarg in
-            let uc_n = String.uppercase n in
+            let uc_n = String.uppercase_ascii n in
             pr "    OPTARG_IF_SET (%d, \"%s\",\n" optarg_index n;
             pr "      optargs_s.bitmask |= %s_%s_BITMASK;\n"
               c_optarg_prefix uc_n;
@@ -646,11 +652,12 @@ get_string_list (lua_State *L, int index)
   const size_t len = lua_objlen (L, index);
   size_t i;
   char **strs;
+  char err[256];
 
   strs = malloc ((len+1) * sizeof (char *));
   if (strs == NULL) {
-    luaL_error (L, \"get_string_list: malloc failed: %%s\",
-                strerror (errno));
+    strerror_r (errno, err, sizeof err);
+    luaL_error (L, \"get_string_list: malloc failed: %%s\", err);
     /*NOTREACHED*/
     return NULL;
   }
