@@ -1,5 +1,5 @@
 (* libguestfs
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 open Printf
 
+open Common_utils
 open Types
 open Utils
 open Pr
@@ -93,16 +94,12 @@ let rec generate_c_api_tests () =
   pr "}\n";
   pr "\n";
 
-  (* Generate the actual tests.  Note that we generate the tests
-   * in reverse order, deliberately, so that (in general) the
-   * newest tests run first.  This makes it quicker and easier to
-   * debug them.
-   *)
+  (* Generate the actual tests. *)
   let test_names =
     List.map (
       fun { name = name; optional = optional; tests = tests } ->
         mapi (generate_one_test name optional) tests
-    ) (List.rev actions) in
+    ) (actions |> sort) in
   let test_names = List.concat test_names in
 
   let nr_tests = List.length test_names in
@@ -201,7 +198,9 @@ static int
   return 0;
 }
 
-" test_name name (String.uppercase test_name) (String.uppercase name);
+" test_name name
+     (String.uppercase_ascii test_name)
+     (String.uppercase_ascii name);
 
   if not_disabled then (
     generate_test_perform name i test_name test;
@@ -381,10 +380,7 @@ and generate_test_command_call ?(expect_error = false) ?(do_return = true) ?test
     match cmd with [] -> assert false | name :: args -> name, args in
 
   (* Look up the function. *)
-  let f =
-    try List.find (fun { name = n } -> n = name) actions
-    with Not_found ->
-      failwithf "%s: in test, command %s was not found" test_name name in
+  let f = Actions.find name in
 
   (* Look up the arguments and return type. *)
   let style_ret, style_args, style_optargs = f.style in
@@ -441,7 +437,7 @@ and generate_test_command_call ?(expect_error = false) ?(do_return = true) ?test
     | StringList _, arg, sym
     | DeviceList _, arg, sym
     | FilenameList _, arg, sym ->
-      let strs = string_split " " arg in
+      let strs = String.nsplit " " arg in
       iteri (
         fun i str ->
           pr "  const char *%s_%d = \"%s\";\n" sym i (c_quote str);
@@ -489,7 +485,7 @@ and generate_test_command_call ?(expect_error = false) ?(do_return = true) ?test
           | OStringList n, "" ->
             pr "  const char *const %s[1] = { NULL };\n" n; true
           | OStringList n, arg ->
-            let strs = string_split " " arg in
+            let strs = String.nsplit " " arg in
             iteri (
               fun i str ->
                 pr "  const char *%s_%d = \"%s\";\n" n i (c_quote str);
@@ -519,10 +515,10 @@ and generate_test_command_call ?(expect_error = false) ?(do_return = true) ?test
     pr "  CLEANUP_FREE_STRING_LIST char **%s;\n" ret;
   | RStruct (_, typ) ->
     pr "  CLEANUP_FREE_%s struct guestfs_%s *%s;\n"
-      (String.uppercase typ) typ ret
+      (String.uppercase_ascii typ) typ ret
   | RStructList (_, typ) ->
     pr "  CLEANUP_FREE_%s_LIST struct guestfs_%s_list *%s;\n"
-      (String.uppercase typ) typ ret
+      (String.uppercase_ascii typ) typ ret
   | RBufferOut _ ->
     pr "  CLEANUP_FREE char *%s;\n" ret;
     pr "  size_t size;\n"
