@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2007-2016 Free Software Foundation, Inc.
+ * Copyright (C) 2004, 2007-2017 Free Software Foundation, Inc.
  * Written by Bruno Haible and Eric Blake
  *
  * This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -26,15 +26,10 @@ SIGNATURE_CHECK (memmem, void *, (void const *, size_t, void const *, size_t));
 #include <stdlib.h>
 #include <unistd.h>
 
+#include "null-ptr.h"
 #include "zerosize-ptr.h"
 #include "macros.h"
 
-static void *
-null_ptr (void)
-{
-  return NULL;
-}
-
 int
 main (int argc, char *argv[])
 {
