@@ -1,5 +1,5 @@
 (* Common utilities for OCaml tools in libguestfs.
- * Copyright (C) 2010-2016 Red Hat Inc.
+ * Copyright (C) 2010-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *)
 
+(* The parts between <stdlib>..</stdlib> are copied into the
+ * generator/common_utils.ml file.  These parts must ONLY use
+ * functions from the OCaml stdlib.
+ *)
+(*<stdlib>*)
+
 module Char : sig
     type t = char
     val chr : int -> char
@@ -26,6 +32,21 @@ module Char : sig
 
     val lowercase_ascii : char -> char
     val uppercase_ascii : char -> char
+
+    val isspace : char -> bool
+    (** Return true if char is a whitespace character. *)
+    val isdigit : char -> bool
+    (** Return true if the character is a digit [[0-9]]. *)
+    val isxdigit : char -> bool
+    (** Return true if the character is a hex digit [[0-9a-fA-F]]. *)
+    val isalpha : char -> bool
+    (** Return true if the character is a US ASCII 7 bit alphabetic. *)
+    val isalnum : char -> bool
+    (** Return true if the character is a US ASCII 7 bit alphanumeric. *)
+
+    val hexdigit : char -> int
+    (** Return the value of a hex digit.  If the char is not in
+        the set [[0-9a-fA-F]] then this returns [-1]. *)
 end
 (** Override the Char module from stdlib. *)
 
@@ -53,6 +74,7 @@ module String : sig
 
     val lowercase_ascii : string -> string
     val uppercase_ascii : string -> string
+    val capitalize_ascii : string -> string
 
     val is_prefix : string -> string -> bool
     (** [is_prefix str prefix] returns true if [prefix] is a prefix of [str]. *)
@@ -64,32 +86,59 @@ module String : sig
     val replace : string -> string -> string -> string
     (** [replace str s1 s2] replaces all instances of [s1] appearing in
         [str] with [s2]. *)
-    val nsplit : string -> string -> string list
-    (** [nsplit sep str] splits [str] into multiple strings at each
-        separator [sep]. *)
+    val replace_char : string -> char -> char -> string
+    (** Replace character in string. *)
     val split : string -> string -> string * string
     (** [split sep str] splits [str] at the first occurrence of the
         separator [sep], returning the part before and the part after.
         If separator is not found, return the whole string and an
         empty string. *)
+    val nsplit : ?max:int -> string -> string -> string list
+    (** [nsplit ?max sep str] splits [str] into multiple strings at each
+        separator [sep].
+
+        As with the Perl split function, you can give an optional
+        [?max] parameter to limit the number of strings returned.  The
+        final element of the list will contain the remainder of the
+        input string. *)
     val lines_split : string -> string list
     (** [lines_split str] splits [str] into lines, keeping continuation
         characters (i.e. [\] at the end of lines) into account. *)
     val random8 : unit -> string
     (** Return a string of 8 random printable characters. *)
+    val triml : ?test:(char -> bool) -> string -> string
+    (** Trim left. *)
+    val trimr : ?test:(char -> bool) -> string -> string
+    (** Trim right. *)
+    val trim : ?test:(char -> bool) -> string -> string
+    (** Trim left and right. *)
+    val count_chars : char -> string -> int
+    (** Count number of times the character occurs in string. *)
+    val explode : string -> char list
+    (** Explode a string into a list of characters. *)
+    val map_chars : (char -> 'a) -> string -> 'a list
+    (** Explode string, then map function over the characters. *)
+    val spaces : int -> string
+    (** [spaces n] creates a string of n spaces. *)
 end
 (** Override the String module from stdlib. *)
 
-(** Exception thrown by [which] when the specified executable is not found
-    in [$PATH]. *)
-exception Executable_not_found of string (* executable *)
-
 val ( // ) : string -> string -> string
 (** Concatenate directory and filename. *)
 
 val quote : string -> string
 (** Shell-safe quoting of a string (alias for {!Filename.quote}). *)
 
+val subdirectory : string -> string -> string
+(** [subdirectory parent path] returns subdirectory part of [path] relative
+    to the [parent]. If [path] and [parent] point to the same directory empty
+    string is returned.
+
+    Note: path normalization on arguments is {b not} performed!
+
+    If [parent] is not a path prefix of [path] the function raises
+    [Invalid_argument]. *)
+
 val ( +^ ) : int64 -> int64 -> int64
 val ( -^ ) : int64 -> int64 -> int64
 val ( *^ ) : int64 -> int64 -> int64
@@ -98,6 +147,8 @@ val ( &^ ) : int64 -> int64 -> int64
 val ( ~^ ) : int64 -> int64
 (** Various int64 operators. *)
 
+external identity : 'a -> 'a = "%identity"
+
 val roundup64 : int64 -> int64 -> int64
 (** [roundup64 i a] returns [i] rounded up to the next multiple of [a]. *)
 val div_roundup64 : int64 -> int64 -> int64
@@ -108,17 +159,16 @@ val int_of_le32 : string -> int64
 val le32_of_int : int64 -> string
 (** Pack a 32 bit integer a 4 byte string stored little endian. *)
 
-val isdigit : char -> bool
-(** Return true if the character is a digit [[0-9]]. *)
-val isxdigit : char -> bool
-(** Return true if the character is a hex digit [[0-9a-fA-F]]. *)
-
 val wrap : ?chan:out_channel -> ?indent:int -> string -> unit
 (** Wrap text. *)
 
 val output_spaces : out_channel -> int -> unit
 (** Write [n] spaces to [out_channel]. *)
 
+val (|>) : 'a -> ('a -> 'b) -> 'b
+(** Added in OCaml 4.01, we can remove our definition when we
+    can assume this minimum version of OCaml. *)
+
 val dropwhile : ('a -> bool) -> 'a list -> 'a list
 (** [dropwhile f xs] drops leading elements from [xs] until
     [f] returns false. *)
@@ -131,6 +181,10 @@ val takewhile : ('a -> bool) -> 'a list -> 'a list
 val filter_map : ('a -> 'b option) -> 'a list -> 'b list
 (** [filter_map f xs] applies [f] to each element of [xs].  If
     [f x] returns [Some y] then [y] is added to the returned list. *)
+val find_map : ('a -> 'b option) -> 'a list -> 'b
+(** [find_map f xs] applies [f] to each element of [xs] until
+    [f x] returns [Some y].  It returns [y].  If we exhaust the
+    list then this raises [Not_found]. *)
 val iteri : (int -> 'a -> 'b) -> 'a list -> unit
 (** [iteri f xs] calls [f i x] for each element, with [i] counting from [0]. *)
 val mapi : (int -> 'a -> 'b) -> 'a list -> 'b list
@@ -194,6 +248,9 @@ val prepend : 'a list -> 'a list ref -> unit
     [prepend] is like {!push_front} above, except it prepends a list
     to the list reference. *)
 
+val unique : unit -> int
+(** Returns a unique number each time called. *)
+
 val may : ('a -> unit) -> 'a option -> unit
 (** [may f (Some x)] runs [f x].  [may f None] does nothing. *)
 
@@ -212,6 +269,18 @@ val protect : f:(unit -> 'a) -> finally:(unit -> unit) -> 'a
     case, but requires a lot more work by the caller.  Perhaps we
     will change this in future.) *)
 
+val failwithf : ('a, unit, string, 'b) format4 -> 'a
+(** Like [failwith] but supports printf-like arguments. *)
+
+exception Executable_not_found of string (* executable *)
+(** Exception thrown by [which] when the specified executable is not found
+    in [$PATH]. *)
+
+val which : string -> string
+(** Return the full path of the specified executable from [$PATH].
+
+    Throw [Executable_not_found] if not available. *)
+
 val prog : string
 (** The program name (derived from {!Sys.executable_name}). *)
 
@@ -224,6 +293,8 @@ val verbose : unit -> bool
 (** Stores the quiet ([--quiet]), trace ([-x]) and verbose ([-v]) flags
     in global variables. *)
 
+(*</stdlib>*)
+
 val message : ('a, unit, string, unit) format4 -> 'a
 (** Timestamped progress messages.  Used for ordinary messages when
     not [--quiet]. *)
@@ -256,9 +327,37 @@ val run_main_and_handle_errors : (unit -> unit) -> unit
 val generated_by : string
 (** The string ["generated by <prog> <version>"]. *)
 
+val virt_tools_data_dir : unit -> string
+(** Parse the [$VIRT_TOOLS_DATA_DIR] environment variable (used by
+    virt-customize and virt-v2v to store auxiliarly tools).  If
+    the environment variable is not set, a default value is
+    calculated based on configure settings. *)
+
+(*<stdlib>*)
+
+val with_open_in : string -> (in_channel -> 'a) -> 'a
+(** [with_open_in filename f] calls function [f] with [filename]
+    open for input.  The file is always closed either on normal
+    return or if the function [f] throws an exception, so this is
+    both safer and more concise than the regular function. *)
+
+val with_open_out : string -> (out_channel -> 'a) -> 'a
+(** [with_open_out filename f] calls function [f] with [filename]
+    open for output.  The file is always closed either on normal
+    return or if the function [f] throws an exception, so this is
+    both safer and more concise than the regular function. *)
+
+val with_openfile : string -> Unix.open_flag list -> Unix.file_perm -> (Unix.file_descr -> 'a) -> 'a
+(** [with_openfile] calls function [f] with [filename] opened by the
+    {!Unix.openfile} function.  The file is always closed either on
+    normal return or if the function [f] throws an exception, so this
+    is both safer and more concise than the regular function. *)
+
 val read_whole_file : string -> string
 (** Read in the whole file as a string. *)
 
+(*</stdlib>*)
+
 val parse_size : string -> int64
 (** Parse a size field, eg. [10G] *)
 
@@ -278,6 +377,8 @@ val create_standard_options : Getopt.speclist -> ?anon_fun:Getopt.anon_fun -> ?k
 
     Returns a new [Getopt.t] handle. *)
 
+(*<stdlib>*)
+
 val compare_version : string -> string -> int
 (** Compare two version strings. *)
 
@@ -288,6 +389,8 @@ val stringify_args : string list -> string
 (** Create a "pretty-print" representation of a program invocation
     (i.e. executable and its arguments). *)
 
+(*</stdlib>*)
+
 val external_command : ?echo_cmd:bool -> string -> string list
 (** Run an external command, slurp up the output as a list of lines.
 
@@ -309,9 +412,13 @@ val shell_command : ?echo_cmd:bool -> string -> int
 val uuidgen : unit -> string
 (** Run uuidgen to return a random UUID. *)
 
+(*<stdlib>*)
+
 val unlink_on_exit : string -> unit
 (** Unlink a temporary file on exit. *)
 
+(*</stdlib>*)
+
 val rmdir_on_exit : string -> unit
 (** Remove a temporary directory on exit (using [rm -rf]). *)
 
@@ -347,15 +454,21 @@ val debug_augeas_errors : Guestfs.guestfs -> unit
 val detect_file_type : string -> [`GZip | `Tar | `XZ | `Zip | `Unknown]
 (** Detect type of a file (for a very limited range of file types). *)
 
+(*<stdlib>*)
+
 val is_block_device : string -> bool
 val is_char_device : string -> bool
 val is_directory : string -> bool
 (** These don't throw exceptions, unlike the [Sys] functions. *)
 
+(*</stdlib>*)
+
 val is_partition : string -> bool
 (** Return true if the host device [dev] is a partition.  If it's
     anything else, or missing, returns false. *)
 
+(*<stdlib>*)
+
 val absolute_path : string -> string
 (** Convert any path to an absolute path. *)
 
@@ -374,16 +487,22 @@ val guest_arch_compatible : string -> bool
 (** Are guest arch and host_cpu compatible, in terms of being able
     to run commands in the libguestfs appliance? *)
 
+val unix_like : string -> bool
+(** Is the guest OS "Unix-like"?  Call this with the result of
+    {!Guestfs.inspect_get_type}. *)
+
 val last_part_of : string -> char -> string option
 (** Return the last part of a string, after the specified separator. *)
 
 val read_first_line_from_file : string -> string
 (** Read only the first line (i.e. until the first newline character)
-    of a file. *)
+    of a file.  If the file is empty this returns an empty string. *)
 
 val is_regular_file : string -> bool
 (** Checks whether the file is a regular file. *)
 
+(*</stdlib>*)
+
 val inspect_mount_root : Guestfs.guestfs -> ?mount_opts_fn:(string -> string) -> string -> unit
 (** Mounts all the mount points of the specified root, just like
     [guestfish -i] does.
@@ -398,11 +517,6 @@ val inspect_mount_root_ro : Guestfs.guestfs -> string -> unit
 val is_btrfs_subvolume : Guestfs.guestfs -> string -> bool
 (** Checks if a filesystem is a btrfs subvolume. *)
 
-val which : string -> string
-(** Return the full path of the specified executable from [$PATH].
-
-    Throw [Executable_not_found] if not available. *)
-
 val inspect_decrypt : Guestfs.guestfs -> unit
 (** Simple implementation of decryption: look for any [crypto_LUKS]
     partitions and decrypt them, then rescan for VGs.  This only works
