@@ -1,5 +1,5 @@
 /* virt-p2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,25 +82,20 @@ xmlBufferDetach (xmlBufferPtr buf)
 }
 #endif
 
-/* How long to wait for qemu-nbd to start (seconds). */
-#define WAIT_QEMU_NBD_TIMEOUT 10
-
 /* Data per NBD connection / physical disk. */
 struct data_conn {
   mexp_h *h;                /* miniexpect handle to ssh */
-  pid_t nbd_pid;            /* qemu pid */
-  int nbd_local_port;       /* local NBD port on physical machine */
+  pid_t nbd_pid;            /* NBD server PID */
   int nbd_remote_port;      /* remote NBD port on conversion server */
 };
 
-static pid_t start_qemu_nbd (int nbd_local_port, const char *device);
-static int wait_qemu_nbd (int nbd_local_port, int timeout_seconds);
 static void cleanup_data_conns (struct data_conn *data_conns, size_t nr);
 static void generate_name (struct config *, const char *filename);
 static void generate_libvirt_xml (struct config *, struct data_conn *, const char *filename);
 static void generate_wrapper_script (struct config *, const char *remote_dir, const char *filename);
-static void generate_dmesg_file (const char *filename);
+static void generate_system_data (const char *dmesg_file, const char *lscpu_file, const char *lspci_file, const char *lsscsi_file, const char *lsusb_file);
 static const char *map_interface_to_network (struct config *, const char *interface);
+static void generate_p2v_version_file (const char *p2v_version_file);
 static void print_quoted (FILE *fp, const char *s);
 
 static char *conversion_error;
@@ -208,6 +203,11 @@ start_conversion (struct config *config,
   char libvirt_xml_file[] = "/tmp/p2v.XXXXXX/physical.xml";
   char wrapper_script[]   = "/tmp/p2v.XXXXXX/virt-v2v-wrapper.sh";
   char dmesg_file[]       = "/tmp/p2v.XXXXXX/dmesg";
+  char lscpu_file[]       = "/tmp/p2v.XXXXXX/lscpu";
+  char lspci_file[]       = "/tmp/p2v.XXXXXX/lspci";
+  char lsscsi_file[]      = "/tmp/p2v.XXXXXX/lsscsi";
+  char lsusb_file[]       = "/tmp/p2v.XXXXXX/lsusb";
+  char p2v_version_file[] = "/tmp/p2v.XXXXXX/p2v-version";
   int inhibit_fd = -1;
 
 #if DEBUG_STDERR
@@ -232,33 +232,15 @@ start_conversion (struct config *config,
   for (i = 0; config->disks[i] != NULL; ++i) {
     data_conns[i].h = NULL;
     data_conns[i].nbd_pid = 0;
-    data_conns[i].nbd_local_port = -1;
     data_conns[i].nbd_remote_port = -1;
   }
 
-  /* Start the data connections and qemu-nbd processes, one per disk. */
+  /* Start the data connections and NBD server processes, one per disk. */
   for (i = 0; config->disks[i] != NULL; ++i) {
+    const char *nbd_local_ipaddr;
+    int nbd_local_port;
     CLEANUP_FREE char *device = NULL;
 
-    if (notify_ui) {
-      CLEANUP_FREE char *msg;
-      if (asprintf (&msg,
-                    _("Opening data connection for %s ..."),
-                    config->disks[i]) == -1)
-        error (EXIT_FAILURE, errno, "asprintf");
-      notify_ui (NOTIFY_STATUS, msg);
-    }
-
-    data_conns[i].h = open_data_connection (config,
-                                            &data_conns[i].nbd_local_port,
-                                            &data_conns[i].nbd_remote_port);
-    if (data_conns[i].h == NULL) {
-      const char *err = get_ssh_error ();
-
-      set_conversion_error ("could not open data connection over SSH to the conversion server: %s", err);
-      goto out;
-    }
-
     if (config->disks[i][0] == '/') {
       device = strdup (config->disks[i]);
       if (!device) {
@@ -273,23 +255,58 @@ start_conversion (struct config *config,
       exit (EXIT_FAILURE);
     }
 
+    if (notify_ui) {
+      CLEANUP_FREE char *msg;
+      if (asprintf (&msg,
+                    _("Starting local NBD server for %s ..."),
+                    config->disks[i]) == -1)
+        error (EXIT_FAILURE, errno, "asprintf");
+      notify_ui (NOTIFY_STATUS, msg);
+    }
+
+    /* Start NBD server listening on the given port number. */
+    data_conns[i].nbd_pid =
+      start_nbd_server (&nbd_local_ipaddr, &nbd_local_port, device);
+    if (data_conns[i].nbd_pid == 0) {
+      set_conversion_error ("NBD server error: %s", get_nbd_error ());
+      goto out;
+    }
+
+    /* Wait for NBD server to start up and listen. */
+    if (wait_for_nbd_server_to_start (nbd_local_ipaddr, nbd_local_port) == -1) {
+      set_conversion_error ("NBD server error: %s", get_nbd_error ());
+      goto out;
+    }
+
+    if (notify_ui) {
+      CLEANUP_FREE char *msg;
+      if (asprintf (&msg,
+                    _("Opening data connection for %s ..."),
+                    config->disks[i]) == -1)
+        error (EXIT_FAILURE, errno, "asprintf");
+      notify_ui (NOTIFY_STATUS, msg);
+    }
+
+    /* Open the SSH data connection, with reverse port forwarding
+     * back to the NBD server.
+     */
+    data_conns[i].h = open_data_connection (config,
+                                            nbd_local_ipaddr, nbd_local_port,
+                                            &data_conns[i].nbd_remote_port);
+    if (data_conns[i].h == NULL) {
+      const char *err = get_ssh_error ();
+
+      set_conversion_error ("could not open data connection over SSH to the conversion server: %s", err);
+      goto out;
+    }
+
 #if DEBUG_STDERR
     fprintf (stderr,
-             "%s: data connection for %s: SSH remote port %d, local port %d\n",
+             "%s: data connection for %s: SSH remote port %d, local port %s:%d\n",
              getprogname (), device,
-             data_conns[i].nbd_remote_port, data_conns[i].nbd_local_port);
+             data_conns[i].nbd_remote_port,
+             nbd_local_ipaddr, nbd_local_port);
 #endif
-
-    /* Start qemu-nbd listening on the given port number. */
-    data_conns[i].nbd_pid =
-      start_qemu_nbd (data_conns[i].nbd_local_port, device);
-    if (data_conns[i].nbd_pid == 0)
-      goto out;
-
-    /* Wait for qemu-nbd to listen */
-    if (wait_qemu_nbd (data_conns[i].nbd_local_port,
-                       WAIT_QEMU_NBD_TIMEOUT) == -1)
-      goto out;
   }
 
   /* Create a remote directory name which will be used for libvirt
@@ -323,12 +340,19 @@ start_conversion (struct config *config,
   memcpy (libvirt_xml_file, tmpdir, strlen (tmpdir));
   memcpy (wrapper_script, tmpdir, strlen (tmpdir));
   memcpy (dmesg_file, tmpdir, strlen (tmpdir));
+  memcpy (lscpu_file, tmpdir, strlen (tmpdir));
+  memcpy (lspci_file, tmpdir, strlen (tmpdir));
+  memcpy (lsscsi_file, tmpdir, strlen (tmpdir));
+  memcpy (lsusb_file, tmpdir, strlen (tmpdir));
+  memcpy (p2v_version_file, tmpdir, strlen (tmpdir));
 
   /* Generate the static files. */
   generate_name (config, name_file);
   generate_libvirt_xml (config, data_conns, libvirt_xml_file);
   generate_wrapper_script (config, remote_dir, wrapper_script);
-  generate_dmesg_file (dmesg_file);
+  generate_system_data (dmesg_file,
+                        lscpu_file, lspci_file, lsscsi_file, lsusb_file);
+  generate_p2v_version_file (p2v_version_file);
 
   /* Open the control connection.  This also creates remote_dir. */
   if (notify_ui)
@@ -357,11 +381,13 @@ start_conversion (struct config *config,
                           wrapper_script, remote_dir, get_ssh_error ());
     goto out;
   }
-  if (scp_file (config, dmesg_file, remote_dir) == -1) {
-    set_conversion_error ("scp: %s to %s: %s",
-                          dmesg_file, remote_dir, get_ssh_error ());
-    goto out;
-  }
+  /* It's not essential that these files are copied. */
+  ignore_value (scp_file (config, dmesg_file, remote_dir));
+  ignore_value (scp_file (config, lscpu_file, remote_dir));
+  ignore_value (scp_file (config, lspci_file, remote_dir));
+  ignore_value (scp_file (config, lsscsi_file, remote_dir));
+  ignore_value (scp_file (config, lsusb_file, remote_dir));
+  ignore_value (scp_file (config, p2v_version_file, remote_dir));
 
   /* Do the conversion.  This runs until virt-v2v exits. */
   if (notify_ui)
@@ -453,224 +479,6 @@ cancel_conversion (void)
   set_cancel_requested (1);
 }
 
-/**
- * Start a local L<qemu-nbd(1)> process.
- *
- * Returns the process ID (E<gt> 0) or C<0> if there is an error.
- */
-static pid_t
-start_qemu_nbd (int port, const char *device)
-{
-  pid_t pid;
-  char port_str[64];
-
-  snprintf (port_str, sizeof port_str, "%d", port);
-
-  pid = fork ();
-  if (pid == -1) {
-    set_conversion_error ("fork: %m");
-    return 0;
-  }
-
-  if (pid == 0) {               /* Child. */
-    close (0);
-    open ("/dev/null", O_RDONLY);
-
-    execlp ("qemu-nbd",
-            "qemu-nbd",
-            "-r",               /* readonly (vital!) */
-            "-p", port_str,     /* listening port */
-            "-t",               /* persistent */
-            "-f", "raw",        /* force raw format */
-            "-b", "localhost",  /* listen only on loopback interface */
-            "--cache=unsafe",   /* use unsafe caching for speed */
-            device,             /* a device like /dev/sda */
-            NULL);
-    perror ("qemu-nbd");
-    _exit (EXIT_FAILURE);
-  }
-
-  /* Parent. */
-  return pid;
-}
-
-static int bind_source_port (int sockfd, int family, int source_port);
-
-/**
- * Connect to C<hostname:dest_port>, resolving the address using
- * L<getaddrinfo(3)>.
- *
- * This also sets the source port of the connection to the first free
- * port number E<ge> C<source_port>.
- *
- * This may involve multiple connections - to IPv4 and IPv6 for
- * instance.
- */
-static int
-connect_with_source_port (const char *hostname, int dest_port, int source_port)
-{
-  struct addrinfo hints;
-  struct addrinfo *results, *rp;
-  char dest_port_str[16];
-  int r, sockfd = -1;
-  int reuseaddr = 1;
-
-  snprintf (dest_port_str, sizeof dest_port_str, "%d", dest_port);
-
-  memset (&hints, 0, sizeof hints);
-  hints.ai_family = AF_UNSPEC;     /* allow IPv4 or IPv6 */
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_flags = AI_NUMERICSERV; /* numeric dest port number */
-  hints.ai_protocol = 0;           /* any protocol */
-
-  r = getaddrinfo (hostname, dest_port_str, &hints, &results);
-  if (r != 0) {
-    set_conversion_error ("getaddrinfo: %s/%s: %s",
-                          hostname, dest_port_str, gai_strerror (r));
-    return -1;
-  }
-
-  for (rp = results; rp != NULL; rp = rp->ai_next) {
-    sockfd = socket (rp->ai_family, rp->ai_socktype, rp->ai_protocol);
-    if (sockfd == -1)
-      continue;
-
-    /* If we run p2v repeatedly (say, running the tests in a loop),
-     * there's a decent chance we'll end up trying to bind() to a port
-     * that is in TIME_WAIT from a prior run.  Handle that gracefully
-     * with SO_REUSEADDR.
-     */
-    if (setsockopt (sockfd, SOL_SOCKET, SO_REUSEADDR,
-                    &reuseaddr, sizeof reuseaddr) == -1)
-      perror ("warning: setsockopt");
-
-    /* Need to bind the source port. */
-    if (bind_source_port (sockfd, rp->ai_family, source_port) == -1) {
-      close (sockfd);
-      sockfd = -1;
-      continue;
-    }
-
-    /* Connect. */
-    if (connect (sockfd, rp->ai_addr, rp->ai_addrlen) == -1) {
-      set_conversion_error ("waiting for qemu-nbd to start: "
-                            "connect to %s/%s: %m",
-                            hostname, dest_port_str);
-      close (sockfd);
-      sockfd = -1;
-      continue;
-    }
-
-    break;
-  }
-
-  freeaddrinfo (results);
-  return sockfd;
-}
-
-static int
-bind_source_port (int sockfd, int family, int source_port)
-{
-  struct addrinfo hints;
-  struct addrinfo *results, *rp;
-  char source_port_str[16];
-  int r;
-
-  snprintf (source_port_str, sizeof source_port_str, "%d", source_port);
-
-  memset (&hints, 0, sizeof (hints));
-  hints.ai_family = family;
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_flags = AI_PASSIVE | AI_NUMERICSERV; /* numeric port number */
-  hints.ai_protocol = 0;                        /* any protocol */
-
-  r = getaddrinfo ("localhost", source_port_str, &hints, &results);
-  if (r != 0) {
-    set_conversion_error ("getaddrinfo (bind): localhost/%s: %s",
-                          source_port_str, gai_strerror (r));
-    return -1;
-  }
-
-  for (rp = results; rp != NULL; rp = rp->ai_next) {
-    if (bind (sockfd, rp->ai_addr, rp->ai_addrlen) == 0)
-      goto bound;
-  }
-
-  set_conversion_error ("waiting for qemu-nbd to start: "
-                        "bind to source port %d: %m",
-                        source_port);
-  freeaddrinfo (results);
-  return -1;
-
- bound:
-  freeaddrinfo (results);
-  return 0;
-}
-
-static int
-wait_qemu_nbd (int nbd_local_port, int timeout_seconds)
-{
-  int sockfd = -1;
-  int result = -1;
-  time_t start_t, now_t;
-  struct timespec half_sec = { .tv_sec = 0, .tv_nsec = 500000000 };
-  struct timeval timeout = { .tv_usec = 0 };
-  char magic[8]; /* NBDMAGIC */
-  size_t bytes_read = 0;
-  ssize_t recvd;
-
-  time (&start_t);
-
-  for (;;) {
-    time (&now_t);
-
-    if (now_t - start_t >= timeout_seconds)
-      goto cleanup;
-
-    /* Source port for probing qemu-nbd should be one greater than
-     * nbd_local_port.  It's not guaranteed to always bind to this port,
-     * but it will hint the kernel to start there and try incrementally
-     * higher ports if needed.  This avoids the case where the kernel
-     * selects nbd_local_port as our source port, and we immediately
-     * connect to ourself.  See:
-     * https://bugzilla.redhat.com/show_bug.cgi?id=1167774#c9
-     */
-    sockfd = connect_with_source_port ("localhost", nbd_local_port,
-                                       nbd_local_port+1);
-    if (sockfd >= 0)
-      break;
-
-    nanosleep (&half_sec, NULL);
-  }
-
-  time (&now_t);
-  timeout.tv_sec = (start_t + timeout_seconds) - now_t;
-  setsockopt (sockfd, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof timeout);
-
-  do {
-    recvd = recv (sockfd, magic, sizeof magic - bytes_read, 0);
-
-    if (recvd == -1) {
-      set_conversion_error ("waiting for qemu-nbd to start: recv: %m");
-      goto cleanup;
-    }
-
-    bytes_read += recvd;
-  } while (bytes_read < sizeof magic);
-
-  if (memcmp (magic, "NBDMAGIC", sizeof magic) != 0) {
-    set_conversion_error ("waiting for qemu-nbd to start: "
-                          "'NBDMAGIC' was not received from qemu-nbd");
-    goto cleanup;
-  }
-
-  result = 0;
- cleanup:
-  close (sockfd);
-
-  return result;
-}
-
 static void
 cleanup_data_conns (struct data_conn *data_conns, size_t nr)
 {
@@ -687,14 +495,14 @@ cleanup_data_conns (struct data_conn *data_conns, size_t nr)
     }
 
     if (data_conns[i].nbd_pid > 0) {
-      /* Kill qemu-nbd process and clean up. */
+      /* Kill NBD process and clean up. */
       kill (data_conns[i].nbd_pid, SIGTERM);
       waitpid (data_conns[i].nbd_pid, NULL, 0);
     }
   }
 }
 
-/* Macros "inspired" by src/launch-libvirt.c */
+/* Macros "inspired" by lib/launch-libvirt.c */
 /* <element */
 #define start_element(element)						\
   if (xmlTextWriterStartElement (xo, BAD_CAST (element)) == -1)         \
@@ -743,10 +551,10 @@ cleanup_data_conns (struct data_conn *data_conns, size_t nr)
   } while (0)
 
 /* An XML comment. */
-#define comment(str)						\
-  do {                                                          \
-    if (xmlTextWriterWriteComment (xo, BAD_CAST (str)) == -1)	\
-      error (EXIT_FAILURE, errno, "xmlTextWriterWriteComment");	\
+#define comment(fs,...)                                                 \
+  do {                                                                  \
+    if (xmlTextWriterWriteFormatComment (xo, fs, ##__VA_ARGS__) == -1) \
+      error (EXIT_FAILURE, errno, "xmlTextWriterWriteFormatComment");   \
   } while (0)
 
 /**
@@ -776,6 +584,8 @@ generate_libvirt_xml (struct config *config, struct data_conn *data_conns,
 
   memkb = config->memory / 1024;
 
+  comment (" %s %s ", getprogname (), PACKAGE_VERSION_FULL);
+
   comment
     (" NOTE!\n"
      "\n"
@@ -1050,6 +860,18 @@ generate_wrapper_script (struct config *config, const char *remote_dir,
   fprintf (fp, "rm -f $log\n");
   fprintf (fp, "\n");
 
+  fprintf (fp,
+           "# Log the environment where virt-v2v will run.\n");
+  fprintf (fp, "printenv > environment\n");
+  fprintf (fp, "\n");
+
+  fprintf (fp,
+           "# Log the version of virt-v2v (for information only).\n");
+  if (config->sudo)
+    fprintf (fp, "sudo -n ");
+  fprintf (fp, "virt-v2v --version > v2v-version\n");
+  fprintf (fp, "\n");
+
   fprintf (fp,
            "# Run virt-v2v.  Send stdout back to virt-p2v.  Send stdout\n"
            "# and stderr (debugging info) to the log file.\n");
@@ -1104,17 +926,48 @@ print_quoted (FILE *fp, const char *s)
 }
 
 /**
- * Put the output of the C<dmesg> command into C<filename>.
+ * Collect data about the system running virt-p2v such as the dmesg
+ * output and lists of PCI devices.  This is useful for diagnosis when
+ * things go wrong.
  *
- * If the command fails, this is non-fatal.
+ * If any command fails, this is non-fatal.
  */
 static void
-generate_dmesg_file (const char *filename)
+generate_system_data (const char *dmesg_file,
+                      const char *lscpu_file,
+                      const char *lspci_file,
+                      const char *lsscsi_file,
+                      const char *lsusb_file)
 {
   CLEANUP_FREE char *cmd = NULL;
 
-  if (asprintf (&cmd, "dmesg >%s 2>&1", filename) == -1)
+  if (asprintf (&cmd,
+                "dmesg >%s 2>&1; "
+                "lscpu >%s 2>&1; "
+                "lspci -vvv >%s 2>&1; "
+                "lsscsi -v >%s 2>&1; "
+                "lsusb -v >%s 2>&1",
+                dmesg_file, lscpu_file, lspci_file, lsscsi_file, lsusb_file)
+      == -1)
     error (EXIT_FAILURE, errno, "asprintf");
 
   ignore_value (system (cmd));
 }
+
+/**
+ * Generate a file containing the version of virt-p2v.
+ *
+ * The version of virt-v2v is contained in the conversion log.
+ */
+static void
+generate_p2v_version_file (const char *p2v_version_file)
+{
+  FILE *fp = fopen (p2v_version_file, "w");
+  if (fp == NULL) {
+    perror (p2v_version_file);
+    return;                     /* non-fatal */
+  }
+  fprintf (fp, "%s %s\n",
+           getprogname (), PACKAGE_VERSION_FULL);
+  fclose (fp);
+}
