@@ -1,5 +1,5 @@
 /* virt-p2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,10 +31,10 @@
  * Once we start conversion, we will open a control connection to send
  * the libvirt configuration data and to start up virt-v2v, and we
  * will open up one data connection per local hard disk.  The data
- * connection(s) have a reverse port forward to the local
- * L<qemu-nbd(8)> server which is serving the content of that hard
- * disk.  The remote port for each data connection is assigned by ssh.
- * See C<open_data_connection> and C<start_remote_conversion>.
+ * connection(s) have a reverse port forward to the local NBD server
+ * which is serving the content of that hard disk.  The remote port
+ * for each data connection is assigned by ssh.  See
+ * C<open_data_connection> and C<start_remote_conversion>.
  */
 
 #include <config.h>
@@ -96,8 +96,9 @@ get_ssh_error (void)
 }
 
 /* Like set_ssh_error, but for errors that aren't supposed to happen. */
-#define set_ssh_internal_error(fs, ...) \
-  set_ssh_error ("internal error: " fs, ##__VA_ARGS__)
+#define set_ssh_internal_error(fs, ...)				   \
+  set_ssh_error ("%s:%d: internal error: " fs, __FILE__, __LINE__, \
+		 ##__VA_ARGS__)
 #define set_ssh_mexp_error(fn) \
   set_ssh_internal_error ("%s: %m", fn)
 #define set_ssh_pcre_error() \
@@ -136,6 +137,7 @@ compile_regexps (void)
    * matching, so fail if that is true here.  In pcre >= 8, all
    * regexps can be used in a partial match.
    */
+#ifdef PCRE_INFO_OKPARTIAL
 #define CHECK_PARTIAL_OK(pattern, re)					\
   do {									\
     pcre_fullinfo ((re), NULL, PCRE_INFO_OKPARTIAL, &p);		\
@@ -146,6 +148,9 @@ compile_regexps (void)
       abort ();								\
     }									\
   } while (0)
+#else
+#define CHECK_PARTIAL_OK(pattern, re) /* skip check */
+#endif
 
 #define COMPILE(re,pattern,options)                                     \
   do {                                                                  \
@@ -158,15 +163,21 @@ compile_regexps (void)
   } while (0)
 
   COMPILE (password_re, "password:", 0);
-  COMPILE (ssh_message_re, "(ssh: .*)", 0);
+  /* Note that (?:.)* is required in order to work around a problem
+   * with partial matching and PCRE in RHEL 5.
+   */
+  COMPILE (ssh_message_re, "(ssh: (?:.)*)", 0);
   COMPILE (sudo_password_re, "sudo: a password is required", 0);
   /* The magic synchronization strings all match this expression.  See
    * start_ssh function below.
    */
   COMPILE (prompt_re,
 	   "###((?:[0123456789abcdefghijklmnopqrstuvwxyz]){8})### ", 0);
+  /* Note that (?:.)* is required in order to work around a problem
+   * with partial matching and PCRE in RHEL 5.
+   */
   COMPILE (version_re,
-           "virt-v2v ([1-9].*)",
+           "virt-v2v ([1-9](?:.)*)",
 	   0);
   COMPILE (feature_libguestfs_rewrite_re, "libguestfs-rewrite", 0);
   COMPILE (feature_colours_option_re, "colours-option", 0);
@@ -295,6 +306,15 @@ cache_ssh_identity (struct config *config)
   return 0;
 }
 
+/* GCC complains about the argv array in the next function which it
+ * thinks might grow to an unbounded size.  Since we control
+ * extra_args, this is not in fact a problem.
+ */
+#if defined(__GNUC__) && GUESTFS_GCC_VERSION >= 40800 /* gcc >= 4.8.0 */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstack-usage="
+#endif
+
 /**
  * Start ssh subprocess with the standard arguments and possibly some
  * optional arguments.  Also handles authentication.
@@ -303,15 +323,18 @@ static mexp_h *
 start_ssh (unsigned spawn_flags, struct config *config,
            char **extra_args, int wait_prompt)
 {
-  size_t i, j, nr_args, count;
+  size_t i = 0;
+  const size_t MAX_ARGS =
+    64 + (extra_args != NULL ? guestfs_int_count_strings (extra_args) : 0);
+  const char *argv[MAX_ARGS];
   char port_str[64];
   char connect_timeout_str[128];
-  CLEANUP_FREE /* [sic] */ const char **args = NULL;
   mexp_h *h;
   const int ovecsize = 12;
   int ovector[ovecsize];
   int saved_timeout;
   int using_password_auth;
+  size_t count;
 
   if (cache_ssh_identity (config) == -1)
     return NULL;
@@ -319,63 +342,52 @@ start_ssh (unsigned spawn_flags, struct config *config,
   /* Are we using password or identity authentication? */
   using_password_auth = config->identity_file == NULL;
 
-  /* Create the ssh argument array. */
-  nr_args = 0;
-  if (extra_args != NULL)
-    nr_args = guestfs_int_count_strings (extra_args);
-
-  if (using_password_auth)
-    nr_args += 13;
-  else
-    nr_args += 15;
-  args = malloc (sizeof (char *) * nr_args);
-  if (args == NULL)
-    error (EXIT_FAILURE, errno, "malloc");
-
-  j = 0;
-  args[j++] = "ssh";
-  args[j++] = "-p";             /* Port. */
+  ADD_ARG (argv, i, "ssh");
+  ADD_ARG (argv, i, "-p");      /* Port. */
   snprintf (port_str, sizeof port_str, "%d", config->port);
-  args[j++] = port_str;
-  args[j++] = "-l";             /* Username. */
-  args[j++] = config->username ? config->username : "root";
-  args[j++] = "-o";             /* Host key will always be novel. */
-  args[j++] = "StrictHostKeyChecking=no";
-  args[j++] = "-o";             /* ConnectTimeout */
+  ADD_ARG (argv, i, port_str);
+  ADD_ARG (argv, i, "-l");      /* Username. */
+  ADD_ARG (argv, i, config->username ? config->username : "root");
+  ADD_ARG (argv, i, "-o");      /* Host key will always be novel. */
+  ADD_ARG (argv, i, "StrictHostKeyChecking=no");
+  ADD_ARG (argv, i, "-o");      /* ConnectTimeout */
   snprintf (connect_timeout_str, sizeof connect_timeout_str,
             "ConnectTimeout=%d", SSH_TIMEOUT);
-  args[j++] = connect_timeout_str;
+  ADD_ARG (argv, i, connect_timeout_str);
+  ADD_ARG (argv, i, "-o");      /* Send ping packets every 5 mins to sshd. */
+  ADD_ARG (argv, i, "ServerAliveInterval=300");
+  ADD_ARG (argv, i, "-o");
+  ADD_ARG (argv, i, "ServerAliveCountMax=6");
   if (using_password_auth) {
     /* Only use password authentication. */
-    args[j++] = "-o";
-    args[j++] = "PreferredAuthentications=keyboard-interactive,password";
+    ADD_ARG (argv, i, "-o");
+    ADD_ARG (argv, i, "PreferredAuthentications=keyboard-interactive,password");
   }
   else {
     /* Use identity file (private key). */
-    args[j++] = "-o";
-    args[j++] = "PreferredAuthentications=publickey";
-    args[j++] = "-i";
-    args[j++] = config->identity_file;
+    ADD_ARG (argv, i, "-o");
+    ADD_ARG (argv, i, "PreferredAuthentications=publickey");
+    ADD_ARG (argv, i, "-i");
+    ADD_ARG (argv, i, config->identity_file);
   }
   if (extra_args != NULL) {
-    for (i = 0; extra_args[i] != NULL; ++i)
-      args[j++] = extra_args[i];
+    for (size_t j = 0; extra_args[j] != NULL; ++j)
+      ADD_ARG (argv, i, extra_args[j]);
   }
-  args[j++] = config->server;   /* Conversion server. */
-  args[j++] = NULL;
-  assert (j == nr_args);
+  ADD_ARG (argv, i, config->server); /* Conversion server. */
+  ADD_ARG (argv, i, NULL);
 
 #if DEBUG_STDERR
   fputs ("ssh command: ", stderr);
-  for (i = 0; i < nr_args - 1; ++i) {
+  for (i = 0; argv[i] != NULL; ++i) {
     if (i > 0) fputc (' ', stderr);
-    fputs (args[i], stderr);
+    fputs (argv[i], stderr);
   }
   fputc ('\n', stderr);
 #endif
 
   /* Create the miniexpect handle. */
-  h = mexp_spawnvf (spawn_flags, "ssh", (char **) args);
+  h = mexp_spawnvf (spawn_flags, "ssh", (char **) argv);
   if (h == NULL) {
     set_ssh_internal_error ("ssh: mexp_spawnvf: %m");
     return NULL;
@@ -555,6 +567,10 @@ start_ssh (unsigned spawn_flags, struct config *config,
   return h;
 }
 
+#if defined(__GNUC__) && GUESTFS_GCC_VERSION >= 40800 /* gcc >= 4.8.0 */
+#pragma GCC diagnostic pop
+#endif
+
 /**
  * Upload a file to remote using L<scp(1)>.
  *
@@ -563,16 +579,16 @@ start_ssh (unsigned spawn_flags, struct config *config,
 int
 scp_file (struct config *config, const char *localfile, const char *remotefile)
 {
-  size_t j, nr_args;
+  size_t i = 0;
+  const size_t MAX_ARGS = 64;
+  const char *argv[MAX_ARGS];
   char port_str[64];
   char connect_timeout_str[128];
   CLEANUP_FREE char *remote = NULL;
-  CLEANUP_FREE /* [sic] */ const char **args = NULL;
   mexp_h *h;
   const int ovecsize = 12;
   int ovector[ovecsize];
   int using_password_auth;
-  size_t i;
 
   if (cache_ssh_identity (config) == -1)
     return -1;
@@ -580,58 +596,47 @@ scp_file (struct config *config, const char *localfile, const char *remotefile)
   /* Are we using password or identity authentication? */
   using_password_auth = config->identity_file == NULL;
 
-  /* Create the scp argument array. */
-  if (using_password_auth)
-    nr_args = 12;
-  else
-    nr_args = 14;
-  args = malloc (sizeof (char *) * nr_args);
-  if (args == NULL)
-    error (EXIT_FAILURE, errno, "malloc");
-
-  j = 0;
-  args[j++] = "scp";
-  args[j++] = "-P";             /* Port. */
+  ADD_ARG (argv, i, "scp");
+  ADD_ARG (argv, i, "-P");      /* Port. */
   snprintf (port_str, sizeof port_str, "%d", config->port);
-  args[j++] = port_str;
-  args[j++] = "-o";             /* Host key will always be novel. */
-  args[j++] = "StrictHostKeyChecking=no";
-  args[j++] = "-o";             /* ConnectTimeout */
+  ADD_ARG (argv, i, port_str);
+  ADD_ARG (argv, i, "-o");      /* Host key will always be novel. */
+  ADD_ARG (argv, i, "StrictHostKeyChecking=no");
+  ADD_ARG (argv, i, "-o");      /* ConnectTimeout */
   snprintf (connect_timeout_str, sizeof connect_timeout_str,
             "ConnectTimeout=%d", SSH_TIMEOUT);
-  args[j++] = connect_timeout_str;
+  ADD_ARG (argv, i, connect_timeout_str);
   if (using_password_auth) {
     /* Only use password authentication. */
-    args[j++] = "-o";
-    args[j++] = "PreferredAuthentications=keyboard-interactive,password";
+    ADD_ARG (argv, i, "-o");
+    ADD_ARG (argv, i, "PreferredAuthentications=keyboard-interactive,password");
   }
   else {
     /* Use identity file (private key). */
-    args[j++] = "-o";
-    args[j++] = "PreferredAuthentications=publickey";
-    args[j++] = "-i";
-    args[j++] = config->identity_file;
+    ADD_ARG (argv, i, "-o");
+    ADD_ARG (argv, i, "PreferredAuthentications=publickey");
+    ADD_ARG (argv, i, "-i");
+    ADD_ARG (argv, i, config->identity_file);
   }
-  args[j++] = localfile;
+  ADD_ARG (argv, i, localfile);
   if (asprintf (&remote, "%s@%s:%s",
                 config->username ? config->username : "root",
                 config->server, remotefile) == -1)
     error (EXIT_FAILURE, errno, "asprintf");
-  args[j++] = remote;
-  args[j++] = NULL;
-  assert (j == nr_args);
+  ADD_ARG (argv, i, remote);
+  ADD_ARG (argv, i, NULL);
 
 #if DEBUG_STDERR
   fputs ("scp command: ", stderr);
-  for (i = 0; i < nr_args - 1; ++i) {
+  for (i = 0; argv[i] != NULL; ++i) {
     if (i > 0) fputc (' ', stderr);
-    fputs (args[i], stderr);
+    fputs (argv[i], stderr);
   }
   fputc ('\n', stderr);
 #endif
 
   /* Create the miniexpect handle. */
-  h = mexp_spawnv ("scp", (char **) args);
+  h = mexp_spawnv ("scp", (char **) argv);
   if (h == NULL) {
     set_ssh_internal_error ("scp: mexp_spawnv: %m");
     return -1;
@@ -770,8 +775,8 @@ test_connection (struct config *config)
     switch (mexp_expect (h,
                          (mexp_regexp[]) {
                            { 100, .re = version_re },
-                           { 101, .re = prompt_re },
-                           { 102, .re = sudo_password_re },
+                           { 101, .re = sudo_password_re },
+                           { 102, .re = prompt_re },
                            { 0 }
                          }, ovector, ovecsize)) {
     case 100:                   /* Got version string. */
@@ -783,15 +788,15 @@ test_connection (struct config *config)
 #endif
       break;
 
-    case 101:             /* Got the prompt. */
-      goto end_of_version;
-
-    case 102:
+    case 101:
       set_ssh_error ("sudo for user \"%s\" requires a password.  Edit /etc/sudoers on the conversion server to ensure the \"NOPASSWD:\" option is set for this user.",
                      config->username);
       mexp_close (h);
       return -1;
 
+    case 102:             /* Got the prompt. */
+      goto end_of_version;
+
     case MEXP_EOF:
       set_ssh_unexpected_eof ("\"virt-v2v --version\" output");
       mexp_close (h);
@@ -1027,11 +1032,10 @@ compatible_version (const char *v2v_version)
   return 1;                     /* compatible */
 }
 
-/* The p2v ISO should allow us to open up just about any port. */
-static int nbd_local_port = 50123;
-
 mexp_h *
-open_data_connection (struct config *config, int *local_port, int *remote_port)
+open_data_connection (struct config *config,
+                      const char *local_ipaddr, int local_port,
+                      int *remote_port)
 {
   mexp_h *h;
   char remote_arg[32];
@@ -1044,9 +1048,7 @@ open_data_connection (struct config *config, int *local_port, int *remote_port)
   const int ovecsize = 12;
   int ovector[ovecsize];
 
-  snprintf (remote_arg, sizeof remote_arg, "0:localhost:%d", nbd_local_port);
-  *local_port = nbd_local_port;
-  nbd_local_port++;
+  snprintf (remote_arg, sizeof remote_arg, "0:%s:%d", local_ipaddr, local_port);
 
   h = start_ssh (0, config, (char **) extra_args, 0);
   if (h == NULL)
