@@ -22,7 +22,7 @@ open Common_gettext.Gettext
 
 module G = Guestfs
 
-let net_hostname_perform g root side_effects =
+let net_hostname_perform (g : Guestfs.guestfs) root side_effects =
   let typ = g#inspect_get_type root in
   let distro = g#inspect_get_distro root in
   match typ, distro with
