@@ -20,13 +20,13 @@
 # Fix part-get-bootable to work with missing/unordered partitions.
 
 set -e
-export LANG=C
 
-guestfish=../../fish/guestfish
+$TEST_FUNCTIONS
+skip_if_skipped
 
 rm -f test.output
 
-$guestfish > test.output <<EOF
+guestfish > test.output <<EOF
 scratch 100M
 run
 part-init /dev/sda mbr
@@ -48,7 +48,7 @@ false" ]; then
     exit 1
 fi
 
-$guestfish > test.output <<EOF
+guestfish > test.output <<EOF
 scratch 100M
 run
 part-init /dev/sda mbr
