@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,6 +66,9 @@ let parse_cmdline () =
   let vdsm_vm_uuid = ref None in
   let vdsm_ovf_output = ref None in (* default "." *)
 
+  let vdsm_compat = ref "0.10" in
+  let set_vdsm_compat s = vdsm_compat := s in
+
   let set_string_option_once optname optref arg =
     match !optref with
     | Some _ ->
@@ -122,7 +125,7 @@ let parse_cmdline () =
     | "libvirt" -> output_mode := `Libvirt
     | "disk" | "local" -> output_mode := `Local
     | "null" -> output_mode := `Null
-    | "ovirt" | "rhev" -> output_mode := `RHEV
+    | "ovirt" | "rhv" | "rhev" -> output_mode := `RHV
     | "qemu" -> output_mode := `QEmu
     | "vdsm" -> output_mode := `VDSM
     | s ->
@@ -167,7 +170,7 @@ let parse_cmdline () =
 
   let argspec = [
     [ S 'b'; L"bridge" ],        Getopt.String ("in:out", add_bridge),     s_"Map bridge 'in' to 'out'";
-    [ L"compressed" ], Getopt.Set compressed,     s_"Compress output file";
+    [ L"compressed" ], Getopt.Set compressed,     s_"Compress output file (-of qcow2 only)";
     [ L"dcpath"; L"dcPath" ],  Getopt.String ("path", set_string_option_once "--dcpath" dcpath),
                                             s_"Override dcPath (for vCenter)";
     [ L"debug-overlay"; L"debug-overlays" ], Getopt.Set debug_overlays, s_"Save overlay files";
@@ -198,6 +201,7 @@ let parse_cmdline () =
     [ L"print-source" ], Getopt.Set print_source, s_"Print source and stop";
     [ L"qemu-boot" ], Getopt.Set qemu_boot,       s_"Boot in qemu (-o qemu only)";
     [ L"root" ],    Getopt.String ("ask|... ", set_root_choice), s_"How to choose root filesystem";
+    [ L"vdsm-compat" ], Getopt.Symbol ("0.10|1.1", ["0.10"; "1.1"], set_vdsm_compat), s_"Write qcow2 with compat=0.10|1.1";
     [ L"vdsm-image-uuid" ], Getopt.String ("uuid", add_vdsm_image_uuid), s_"Output image UUID(s)";
     [ L"vdsm-vol-uuid" ], Getopt.String ("uuid", add_vdsm_vol_uuid), s_"Output vol UUID(s)";
     [ L"vdsm-vm-uuid" ], Getopt.String ("uuid", set_string_option_once "--vdsm-vm-uuid" vdsm_vm_uuid),
@@ -216,7 +220,7 @@ let parse_cmdline () =
  virt-v2v -ic vpx://vcenter.example.com/Datacenter/esxi -os imported esx_guest
 
  virt-v2v -ic vpx://vcenter.example.com/Datacenter/esxi esx_guest \
-   -o rhev -os rhev.nfs:/export_domain --network rhevm
+   -o rhv -os rhv.nfs:/export_domain --network ovirtmgmt
 
  virt-v2v -i libvirtxml guest-domain.xml -o local -os /var/tmp
 
@@ -259,6 +263,7 @@ read the man page virt-v2v(1).
   let print_source = !print_source in
   let qemu_boot = !qemu_boot in
   let root_choice = !root_choice in
+  let vdsm_compat = !vdsm_compat in
   let vdsm_image_uuids = List.rev !vdsm_image_uuids in
   let vdsm_vol_uuids = List.rev !vdsm_vol_uuids in
   let vdsm_vm_uuid = !vdsm_vm_uuid in
@@ -271,7 +276,11 @@ read the man page virt-v2v(1).
   if args = [] && machine_readable then (
     printf "virt-v2v\n";
     printf "libguestfs-rewrite\n";
+    printf "vcenter-https\n";
+    printf "xen-ssh\n";
     printf "colours-option\n";
+    printf "vdsm-compat-option\n";
+    printf "in-place\n";
     List.iter (printf "input:%s\n") (Modules_list.input_modules ());
     List.iter (printf "output:%s\n") (Modules_list.output_modules ());
     List.iter (printf "convert:%s\n") (Modules_list.convert_modules ());
@@ -328,31 +337,38 @@ read the man page virt-v2v(1).
           error (f_"expecting an OVA file name on the command line") in
       Input_ova.input_ova filename in
 
+  (* Common error message. *)
+  let error_option_cannot_be_used_in_output_mode mode opt =
+    error (f_"-o %s: %s option cannot be used in this output mode") mode opt
+  in
+
   (* Parse the output mode. *)
   if output_mode <> `Not_set && in_place then
     error (f_"-o and --in-place cannot be used at the same time");
-  let output =
+  let output, output_format, output_alloc =
     match output_mode with
     | `Glance ->
       if output_conn <> None then
-        error (f_"-o glance: -oc option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "glance" "-oc";
       if output_storage <> None then
-        error (f_"-o glance: -os option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "glance" "-os";
       if qemu_boot then
-        error (f_"-o glance: --qemu-boot option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "glance" "--qemu-boot";
       if not do_copy then
-        error (f_"--no-copy and '-o glance' cannot be used at the same time");
-      Output_glance.output_glance ()
+        error_option_cannot_be_used_in_output_mode "glance" "--no-copy";
+      Output_glance.output_glance (),
+      output_format, output_alloc
 
     | `Not_set
     | `Libvirt ->
       let output_storage =
         match output_storage with None -> "default" | Some os -> os in
       if qemu_boot then
-        error (f_"-o libvirt: --qemu-boot option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "libvirt" "--qemu-boot";
       if not do_copy then
-        error (f_"--no-copy and '-o libvirt' cannot be used at the same time");
-      Output_libvirt.output_libvirt output_conn output_storage
+        error_option_cannot_be_used_in_output_mode "libvirt" "--no-copy";
+      Output_libvirt.output_libvirt output_conn output_storage,
+      output_format, output_alloc
 
     | `Local ->
       let os =
@@ -363,17 +379,24 @@ read the man page virt-v2v(1).
            error (f_"-os %s: output directory does not exist or is not a directory") d
         | Some d -> d in
       if qemu_boot then
-        error (f_"-o local: --qemu-boot option cannot be used in this output mode");
-      Output_local.output_local os
+        error_option_cannot_be_used_in_output_mode "local" "--qemu-boot";
+      Output_local.output_local os,
+      output_format, output_alloc
 
     | `Null ->
+      if output_alloc <> Sparse then
+        error_option_cannot_be_used_in_output_mode "null" "-oa";
       if output_conn <> None then
-        error (f_"-o null: -oc option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "null" "-oc";
+      if output_format <> None then
+        error_option_cannot_be_used_in_output_mode "null" "-of";
       if output_storage <> None then
-        error (f_"-o null: -os option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "null" "-os";
       if qemu_boot then
-        error (f_"-o null: --qemu-boot option cannot be used in this output mode");
-      Output_null.output_null ()
+        error_option_cannot_be_used_in_output_mode "null" "--qemu-boot";
+      Output_null.output_null (),
+      (* Force output format to raw sparse in -o null mode. *)
+      Some "raw", Sparse
 
     | `QEmu ->
       let os =
@@ -383,17 +406,19 @@ read the man page virt-v2v(1).
         | Some d when not (is_directory d) ->
            error (f_"-os %s: output directory does not exist or is not a directory") d
         | Some d -> d in
-      Output_qemu.output_qemu os qemu_boot
+      Output_qemu.output_qemu os qemu_boot,
+      output_format, output_alloc
 
-    | `RHEV ->
+    | `RHV ->
       let os =
         match output_storage with
         | None ->
-           error (f_"-o rhev: output storage was not specified, use '-os'");
+           error (f_"-o rhv: output storage was not specified, use '-os'");
         | Some d -> d in
       if qemu_boot then
-        error (f_"-o rhev: --qemu-boot option cannot be used in this output mode");
-      Output_rhev.output_rhev os output_alloc
+        error_option_cannot_be_used_in_output_mode "rhv" "--qemu-boot";
+      Output_rhv.output_rhv os output_alloc,
+      output_format, output_alloc
 
     | `VDSM ->
       let os =
@@ -402,7 +427,7 @@ read the man page virt-v2v(1).
            error (f_"-o vdsm: output storage was not specified, use '-os'");
         | Some d -> d in
       if qemu_boot then
-        error (f_"-o vdsm: --qemu-boot option cannot be used in this output mode");
+        error_option_cannot_be_used_in_output_mode "vdsm" "--qemu-boot";
       let vdsm_vm_uuid =
         match vdsm_vm_uuid with
         | None ->
@@ -415,8 +440,10 @@ read the man page virt-v2v(1).
         vol_uuids = vdsm_vol_uuids;
         vm_uuid = vdsm_vm_uuid;
         ovf_output = vdsm_ovf_output;
+        compat = vdsm_compat;
       } in
-      Output_vdsm.output_vdsm os vdsm_params output_alloc in
+      Output_vdsm.output_vdsm os vdsm_params output_alloc,
+      output_format, output_alloc in
 
   {
     compressed = compressed; debug_overlays = debug_overlays;
