@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@ let rec inspect_source root_choice g =
 
   reject_if_not_installed_image g root;
 
+  let typ = g#inspect_get_type root in
+
   (* Mount up the filesystems. *)
   let mps = g#inspect_get_mountpoints root in
   let cmp (a,_) (b,_) = compare (String.length a) (String.length b) in
@@ -65,9 +67,22 @@ let rec inspect_source root_choice g =
       StringMap.add name (app :: vs) map
   ) StringMap.empty apps in
 
+  (* If the guest is Windows, get some Windows-specific inspection
+   * data, else (for simplicity when accessing) use empty strings.
+   *)
+  let systemroot, software_hive, system_hive, current_cs =
+    match typ with
+    | "windows" ->
+       g#inspect_get_windows_systemroot root,
+       g#inspect_get_windows_software_hive root,
+       g#inspect_get_windows_system_hive root,
+       g#inspect_get_windows_current_control_set root
+    | _ ->
+       "", "", "", "" in
+
   let inspect = {
     i_root = root;
-    i_type = g#inspect_get_type root;
+    i_type = typ;
     i_distro = g#inspect_get_distro root;
     i_arch = g#inspect_get_arch root;
     i_major_version = g#inspect_get_major_version root;
@@ -80,6 +95,10 @@ let rec inspect_source root_choice g =
     i_apps = apps;
     i_apps_map = apps_map;
     i_firmware = get_firmware_bootable_device g;
+    i_windows_systemroot = systemroot;
+    i_windows_software_hive = software_hive;
+    i_windows_system_hive = system_hive;
+    i_windows_current_control_set = current_cs;
   } in
   debug "%s" (string_of_inspect inspect);
 
@@ -89,7 +108,7 @@ let rec inspect_source root_choice g =
 
 and choose_root root_choice g = function
   | [] ->
-     error (f_"inspection could not detect the source guest (or physical machine).\n\nAssuming that you are running virt-v2v/virt-p2v on a source which is supported (and not, for example, a blank disk), then this should not happen.  You should run 'virt-v2v -v -x ... >& log' and attach the complete log to a new bug report (see http://libguestfs.org).\n\nNo root device found in this operating system image.");
+     error (f_"inspection could not detect the source guest (or physical machine).\n\nAssuming that you are running virt-v2v/virt-p2v on a source which is supported (and not, for example, a blank disk), then this should not happen.\n\nNo root device found in this operating system image.");
   | [root] -> root (* only one root, so return it *)
   | roots ->
      (* If there are multiple roots, use the [--root] option supplied
@@ -193,5 +212,5 @@ and sanity_check_inspection inspect =
 
 and error_if_unknown fieldname value =
   if value = "unknown" then
-    error (f_"inspection could not detect the source guest (or physical machine).\n\nAssuming that you are running virt-v2v/virt-p2v on a source which is supported (and not, for example, a blank disk), then this should not happen.  You should run 'virt-v2v -v -x ... >& log' and attach the complete log to a new bug report (see http://libguestfs.org).\n\nInspection field '%s' was 'unknown'.")
+    error (f_"inspection could not detect the source guest (or physical machine).\n\nAssuming that you are running virt-v2v/virt-p2v on a source which is supported (and not, for example, a blank disk), then this should not happen.\n\nInspection field '%s' was 'unknown'.")
           fieldname
