@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ open Printf
 
 open Common_gettext.Gettext
 open Common_utils
+open Unix_utils
 
 open Types
 open Utils
@@ -32,17 +33,13 @@ class output_glance () =
    *)
   let tmpdir =
     let base_dir = (open_guestfs ())#get_cachedir () in
-    let t = Mkdtemp.temp_dir ~base_dir "glance." "" in
+    let t = Mkdtemp.temp_dir ~base_dir "glance." in
     rmdir_on_exit t;
     t in
 object
   inherit output
 
-  method as_options = "-o glance"
-
-  method supported_firmware = [ TargetBIOS ]
-
-  method prepare_targets source targets =
+  method precheck () =
     (* This does nothing useful except to check that the user has
      * supplied all the correct auth environment variables to make
      * 'glance' commands work as the current user.  If not then the
@@ -55,8 +52,13 @@ object
     if verbose () then (
       eprintf "version of the glance client:\n%!";
       ignore (shell_command "glance --version");
-    );
+    )
 
+  method as_options = "-o glance"
+
+  method supported_firmware = [ TargetBIOS; TargetUEFI ]
+
+  method prepare_targets source targets =
     (* Write targets to a temporary local file - see above for reason. *)
     List.map (
       fun t ->
@@ -65,10 +67,7 @@ object
     ) targets
 
   method create_metadata source targets _ guestcaps inspect target_firmware =
-    (* See #supported_firmware above. *)
-    assert (target_firmware = TargetBIOS);
-
-    (* The first disk, assumed to be the system, will be called
+    (* The first disk, assumed to be the system disk, will be called
      * "guestname".  Subsequent disks, assumed to be data disks,
      * will be called "guestname-disk2" etc.  The manual strongly
      * hints you should import the data disks to Cinder.
@@ -81,7 +80,7 @@ object
 
         (* Set the properties (ie. metadata). *)
         let min_ram = source.s_memory /^ 1024L /^ 1024L in
-        let properties = [
+        let properties = ref [
           "hw_disk_bus",
           (match guestcaps.gcaps_block_bus with
            | Virtio_blk -> "virtio"
@@ -108,26 +107,33 @@ object
            | x -> x (* everything else is the same in libguestfs and OpenStack*)
           )
         ] in
-        let properties =
-          match guestcaps.gcaps_block_bus with
-          | Virtio_SCSI -> ("hw_scsi_model", "virtio-scsi") :: properties
-          | Virtio_blk | IDE -> properties in
-        let properties =
-          match inspect.i_major_version, inspect.i_minor_version with
-          | 0, 0 -> properties
-          | x, 0 -> ("os_version", string_of_int x) :: properties
-          | x, y -> ("os_version", sprintf "%d.%d" x y) :: properties in
+        (match guestcaps.gcaps_block_bus with
+         | Virtio_SCSI ->
+            push_back properties ("hw_scsi_model", "virtio-scsi")
+         | Virtio_blk | IDE -> ()
+        );
+        (match inspect.i_major_version, inspect.i_minor_version with
+         | 0, 0 -> ()
+         | x, 0 -> push_back properties ("os_version", string_of_int x)
+         | x, y -> push_back properties ("os_version", sprintf "%d.%d" x y)
+        );
+        (match target_firmware with
+         | TargetBIOS -> ()
+         | TargetUEFI ->
+            push_back properties ("hw_firmware_type", "uefi")
+        );
 
+        let properties =
+          List.flatten (
+            List.map (
+              fun (k, v) -> [ "--property"; sprintf "%s=%s" k v ]
+            ) !properties
+          ) in
         let cmd = [ "glance"; "image-create"; "--name"; name;
                     "--disk-format=" ^ target_format;
                     "--container-format=bare"; "--file"; target_file;
                     "--min-ram"; Int64.to_string min_ram ] @
-                  (List.flatten
-                    (List.map (
-                       fun (k, v) ->
-                         [ "--property"; sprintf "%s=%s" k v ]
-                    ) properties
-                  )) in
+                  properties in
         if run_command cmd <> 0 then
           error (f_"glance: image upload to glance failed, see earlier errors");
       ) targets
