@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,15 +20,16 @@ open Unix
 open Printf
 
 open Common_gettext.Gettext
-
-module G = Guestfs
-
 open Common_utils
+open Unix_utils
+
 open Types
 open Utils
 
 open Cmdline
 
+module G = Guestfs
+
 type conversion_mode =
     | Copying of overlay list * target list
     | In_place
@@ -41,8 +42,8 @@ let rec main () =
 
   (* Print the version, easier than asking users to tell us. *)
   debug "%s: %s %s (%s)"
-        prog Guestfs_config.package_name
-        Guestfs_config.package_version Guestfs_config.host_cpu;
+        prog Guestfs_config.package_name Guestfs_config.package_version_full
+        Guestfs_config.host_cpu;
 
   (* Print the libvirt version if debugging.  Note that if
    * we're configured --without-libvirt, then this will throw
@@ -51,11 +52,15 @@ let rec main () =
    *)
   if verbose () then (
     try
-      let major, minor, release = Domainxml.libvirt_get_version () in
+      let major, minor, release = Libvirt_utils.libvirt_get_version () in
       debug "libvirt version: %d.%d.%d" major minor release
     with _ -> ()
   );
 
+  (* Perform pre-flight checks on the input and output objects. *)
+  input#precheck ();
+  output#precheck ();
+
   let source = open_source cmdline input in
   let source = set_source_name cmdline source in
   let source = set_source_networks_and_bridges cmdline source in
@@ -102,7 +107,6 @@ let rec main () =
 
   (* Conversion. *)
   let guestcaps =
-    let keep_serial_console = output#keep_serial_console in
     let rcaps =
       match conversion_mode with
       | Copying _ ->
@@ -110,7 +114,7 @@ let rec main () =
       | In_place ->
          rcaps_from_source source in
 
-    do_convert g inspect source keep_serial_console rcaps in
+    do_convert g inspect source output rcaps in
 
   g#umount_all ();
 
@@ -548,7 +552,7 @@ and check_target_free_space mpstats source targets output =
   output#check_target_free_space source targets
 
 (* Conversion. *)
-and do_convert g inspect source keep_serial_console rcaps =
+and do_convert g inspect source output rcaps =
   (match inspect.i_product_name with
   | "unknown" ->
     message (f_"Converting the guest to run on KVM")
@@ -563,7 +567,8 @@ and do_convert g inspect source keep_serial_console rcaps =
         inspect.i_type inspect.i_distro in
   debug "picked conversion module %s" conversion_name;
   debug "requested caps: %s" (string_of_requested_guestcaps rcaps);
-  let guestcaps = convert ~keep_serial_console g inspect source rcaps in
+  let guestcaps =
+    convert g inspect source (output :> Types.output_settings) rcaps in
   debug "%s" (string_of_guestcaps guestcaps);
 
   (* Did we manage to install virtio drivers? *)
