@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2017 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,17 +19,15 @@
 (** Functions for installing Windows virtio drivers. *)
 
 val install_drivers
-    : Guestfs.guestfs -> Types.inspect -> string -> int64 -> string ->
-      Types.requested_guestcaps ->
+    : Registry.t -> Types.inspect -> Types.requested_guestcaps ->
       Types.guestcaps_block_type * Types.guestcaps_net_type * Types.guestcaps_video_type
-(** [install_drivers g inspect systemroot root current_cs rcaps]
+(** [install_drivers reg inspect rcaps]
     installs virtio drivers from the driver directory or driver
     ISO into the guest driver directory and updates the registry
     so that the [viostor.sys] driver gets loaded by Windows at boot.
 
-    [root] is the root node of the system hive (which is open for writes
-    when this function is called).  [current_cs] is the name of the
-    [CurrentControlSet] (eg. ["ControlSet001"]).
+    [reg] is the system hive which is open for writes when this
+    function is called.
 
     [rcaps] is the set of guest "capabilities" requested by the caller.  This
     may include the type of the block driver, network driver, and video driver.
